/**
 * =============================================================================
 * SM:CTF
 * Capture the Flag gamemode for CS:S
 * Thanks to sn4k3 for his SCTF eventscript.
 * http://forums.eventscripts.com/viewtopic.php?t=25254
 *
 * SM:CTF (C)2011 Jannik Hartung  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Changelog:
 * 1.0 First Release
 * 1.0.1 Added key hint texts with tips. E.g. can't pickup a flag if still protected
 * 1.0.2 Added team of flagcarrier to information panel, highered flag beam up to the sky
 * 1.0.3 Removed hidden bots in courtesy of mp_ignore_round_win_conditions 1
 * 1.0.4 Fixed sm_setflag command to instantly enable the new flag
 *       Fixed class description is shown twice in the menu for classes, which are available for both teams
 *       Fixed flag beams being 10 inches off the ground
 * 
 * 1.1   Added smctf_nadetrails convar to control grenade beam trails
 *       Added countdown display to flag protection
 *       Added smctf_captureownflag and smctf_requireownflaghome convars
 *       Added smctf_resetscores convar to reset scores to 0 if there is no player on the server anymore
 *       Added mp_restartgame handling to reset scores
 *       Fixed not always showing the correct team name when capturing a flag
 *       Fixed weapons not getting removed on death and disconnect.
 *       Fixed not removing all dropped weapons. Tried to avoid removal of currently held weapons.
 *       Fixed unlimited backclip ammo
 *       Fixed not being able to disable player spawn protection
 *       Now restarts the game by the time the 2nd player spawns
 *       Switched to SM1.4's new CS_OnCSWeaponDrop forward to avoid loss of ammo
 */
#pragma semicolon 1
#include <sourcemod>
#include <sdktools>
#include <sdkhooks>
#include <colors>
#include <cstrike>
#include <smlib>
#include <loghelper>

#define PLUGIN_VERSION "1.1"

#define FFADE_IN            0x0001        // Just here so we don't pass 0 into the function
#define FFADE_OUT           0x0002        // Fade out (not in)
#define FFADE_MODULATE      0x0004        // Modulate (don't blend)
#define FFADE_STAYOUT       0x0008        // ignores the duration, stays faded out until new ScreenFade message received
#define FFADE_PURGE         0x0010        // Purges all other fades, replacing them with this one

#define PREFIX "{olive}SM:CTF {default}>{green} "

new bool:g_bRoundEnded = false;

new Handle:g_hRespawnWave[2] = {INVALID_HANDLE, INVALID_HANDLE};
new g_iRespawnWaveTime[2] = {0, 0};
new g_iRespawnWaveCountdownLength[2] = {0, 0};
new Handle:g_hRespawnWaveRules;

new Handle:g_hRemoveDroppedWeapons = INVALID_HANDLE;
new Handle:g_hTimeLimitEnforcer = INVALID_HANDLE;
new g_iTimeLimit = 0;

new Handle:g_hResetSpawnProtection[MAXPLAYERS+1] = {INVALID_HANDLE,...};
new bool:g_bFirstSpawn[MAXPLAYERS+1] = {true,...};

new bool:g_bNadeTrails;

// ConVars
new Handle:g_hCVScoreLimit = INVALID_HANDLE;
new Handle:g_hCVDropResetTime = INVALID_HANDLE;
new Handle:g_hCVScoreForTakeFlag = INVALID_HANDLE;
new Handle:g_hCVScoreForReturnFlag = INVALID_HANDLE;
new Handle:g_hCVScoreForScoreFlag = INVALID_HANDLE;
new Handle:g_hCVScoreForDropFlag = INVALID_HANDLE;
new Handle:g_hCVFlagProtectionTime = INVALID_HANDLE;
new Handle:g_hCVStatPrune = INVALID_HANDLE;
new Handle:g_hCVSpawnProtection = INVALID_HANDLE;
new Handle:g_hCVUnlimitedMagazine = INVALID_HANDLE;
new Handle:g_hCVReturnOnDrop = INVALID_HANDLE;
new Handle:g_hCVReturnOnOwnTeam = INVALID_HANDLE;
new Handle:g_hCVCaptureOwnFlag = INVALID_HANDLE;
new Handle:g_hCVRequireOwnFlagHome = INVALID_HANDLE;
new Handle:g_hCVNadeTrails = INVALID_HANDLE;
new Handle:g_hCVResetScores = INVALID_HANDLE;

/* Thanks to kzfi`p3tsku for this info. */
// To catch the intermission time
new bool:g_bIsIntermissionCalled = false;
new UserMsg:g_VGuiMenu;

new Handle:sv_tags;
new Handle:mp_restartgame;

#include "smctf_stats.sp"
#include "smctf_flag.sp"
#include "smctf_classes.sp"

public Plugin:myinfo = 
{
	name = "SM:CTF Capture the flag",
	author = "Jannik 'Peace-Maker' Hartung",
	description = "Capture the flag gamemode",
	version = PLUGIN_VERSION,
	url = "http://www.wcfan.de/"
}

public OnPluginStart()
{
	new Handle:hVersion = CreateConVar("smctf_version", PLUGIN_VERSION, "Capture the flag modification", FCVAR_PLUGIN|FCVAR_NOTIFY|FCVAR_REPLICATED|FCVAR_DONTRECORD);
	if(hVersion != INVALID_HANDLE)
		SetConVarString(hVersion, PLUGIN_VERSION);
	
	g_hCVScoreLimit = CreateConVar("smctf_scorelimit", "10", "When any team scores x flags game will end.", FCVAR_PLUGIN, true, 1.0);
	g_hCVDropResetTime = CreateConVar("smctf_dropreset", "15", "How many seconds after drop should we return the flag automatically?", FCVAR_PLUGIN, true, 0.0);
	g_hCVScoreForTakeFlag = CreateConVar("smctf_flagtake", "1", "How many kills should a player win when capturing a flag?", FCVAR_PLUGIN, true, 0.0);
	g_hCVScoreForReturnFlag = CreateConVar("smctf_flagreturn", "1", "How many kills should a player win when returning his team's flag to his base?", FCVAR_PLUGIN, true, 0.0);
	g_hCVScoreForScoreFlag = CreateConVar("smctf_flagscore", "2", "How many kills should a player win when scoring?", FCVAR_PLUGIN, true, 0.0);
	g_hCVScoreForDropFlag = CreateConVar("smctf_flagdrop", "1", "How many kills should a player lose when dropping a flag?", FCVAR_PLUGIN, true, 0.0);
	g_hCVFlagProtectionTime = CreateConVar("smctf_flagprotection", "5", "How long should the dropped/returned flag be protected from being picked up in seconds?", FCVAR_PLUGIN, true, 0.1);
	g_hCVStatPrune = CreateConVar("smctf_statprune", "230400", "Delete stats of players who haven't been on the server for x minutes. (Default: 230400 -> 160 days)", FCVAR_PLUGIN, true, 1440.0);
	g_hCVSpawnProtection = CreateConVar("smctf_spawnprotection", "3", "How long should a player resist any damage after spawn?", FCVAR_PLUGIN, true, 0.1);
	g_hCVUnlimitedMagazine = CreateConVar("smctf_unlimitedammo", "1", "Give players ammo back, if they got 0 in their 2nd clip?", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_hCVReturnOnDrop = CreateConVar("smctf_returnondrop", "0", "Instantly return the flag to it's spawn, when it's dropped?", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_hCVReturnOnOwnTeam = CreateConVar("smctf_returnonownteam", "0", "Instantly return the dropped flag to it's spawn, if it gets picked up by a player of the same colored team?", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_hCVCaptureOwnFlag = CreateConVar("smctf_captureownflag", "1", "Are players allowed to capture their own dropped flag? Set this to 0 and the flag is only returned after the timeout.", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_hCVRequireOwnFlagHome = CreateConVar("smctf_requireownflaghome", "1", "Should it only be possible to score, if the own flag is at it's spawn?", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_hCVNadeTrails = CreateConVar("smctf_nadetrails", "1", "Add colored beam trails to grenades?", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_hCVResetScores = CreateConVar("smctf_resetscores", "1", "Reset both team's scores to 0 if there is no player on the server anymore?", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	
	HookConVarChange(g_hCVUnlimitedMagazine, ConVar_OnChanged);
	HookConVarChange(g_hCVNadeTrails, ConVar_OnChanged);
	
	mp_restartgame = FindConVar("mp_restartgame");
	HookConVarChange(mp_restartgame, ConVar_OnRestartGame);
	
	HookEvent("round_start", Event_OnRoundStart);
	HookEvent("round_end", Event_OnRoundEnd);
	HookEvent("player_death", Event_OnPlayerDeath, EventHookMode_Pre);
	HookEvent("player_spawn", Event_OnPlayerSpawn);
	HookEvent("player_team", Event_OnPlayerTeam);
	HookEvent("weapon_fire", Event_OnWeaponFire);
	
	RegConsoleCmd("sm_class", Command_ClassMenu, "Opens the class menu");
	RegConsoleCmd("sm_ctfstats", Command_Stats, "Opens a panel with player stats");
	
	// Additional say hook for ctfstats
	RegConsoleCmd("say", Command_SayHook);
	RegConsoleCmd("say_team", Command_SayHook);
	
	RegAdminCmd("sm_setflag", Command_SetFlag, ADMFLAG_CONFIG, "Set's the position of a flag on a map at the origin of the player. Usage: sm_setflag <red|blue>");
	RegAdminCmd("sm_dropflag", Command_DropFlag, ADMFLAG_BAN, "Forces the a player to drop the flag. Usage: sm_dropflag <#userid|steamid|name>");
	RegAdminCmd("sm_resetctfstats", Command_ResetStats, ADMFLAG_CONFIG, "Resets a players ctf stats. Usage: sm_resetctfstats <#userid|steamid|name>");
	
	LoadTranslations("common.phrases");
	
	g_hRespawnWaveRules = CreateArray(2);
	
	// Used to hook intermission
	g_VGuiMenu = GetUserMessageId("VGUIMenu");
	HookUserMessage(g_VGuiMenu, _VGuiMenu);
	
	// Force the timelimit, even if the round never ends
	new Handle:hTimeLimit = FindConVar("mp_timelimit");
	HookConVarChange(hTimeLimit, ConVar_TimeLimitChanged);
	g_iTimeLimit = GetConVarInt(hTimeLimit)*60;
	
	// Initialize the stats
	InitializeDatabase();
	
	g_iRadarOffset = FindSendPropOffs("CCSPlayerResource", "m_bPlayerSpotted");
	if(g_iRadarOffset == -1)
		SetFailState("Couldnt find the CCSPlayerResource::m_bPlayerSpotted offset!");
	
	g_iPlayerVIPOffset = FindSendPropOffs("CCSPlayerResource", "m_iPlayerVIP");
	if(g_iPlayerVIPOffset == -1)
		SetFailState("Couldnt find the CCSPlayerResource::m_iPlayerVIP offset!");
	
	g_iAmmoOffset = FindSendPropOffs("CBasePlayer", "m_iAmmo");
	if(g_iAmmoOffset == -1)
		SetFailState("Couldnt find the CBasePlayer::m_iAmmo offset!");
	
	
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i))
		{
			OnClientPutInServer(i);
		}
	}
	
	sv_tags = FindConVar("sv_tags");
	MyAddServerTag("ctf");
	
	AutoExecConfig(true, "plugin.smctf");
}

/**
 * Events
 */
public OnMapStart()
{
	AddFileToDownloadsTable("models/mapmodels/flags.dx80.vtx");
	AddFileToDownloadsTable("models/mapmodels/flags.dx90.vtx");
	AddFileToDownloadsTable("models/mapmodels/flags.mdl");
	AddFileToDownloadsTable("models/mapmodels/flags.sw.vtx");
	AddFileToDownloadsTable("models/mapmodels/flags.vvd");
	//AddFileToDownloadsTable("models/mapmodels/flags.xbox.vtx");
	AddFileToDownloadsTable("models/mapmodels/pole.dx80.vtx");
	AddFileToDownloadsTable("models/mapmodels/pole.dx90.vtx");
	AddFileToDownloadsTable("models/mapmodels/pole.mdl");
	AddFileToDownloadsTable("models/mapmodels/pole.sw.vtx");
	AddFileToDownloadsTable("models/mapmodels/pole.vvd");
	AddFileToDownloadsTable("models/mapmodels/pole.xbox.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_blueflag.dx80.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_blueflag.dx90.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_blueflag.mdl");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_blueflag.sw.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_blueflag.vvd");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_blueflag.phy");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_blueflag.xbox.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_redflag.dx80.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_redflag.dx90.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_redflag.mdl");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_redflag.sw.vtx");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_redflag.vvd");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_redflag.phy");
	AddFileToDownloadsTable("models/extras/sctf/info_speech_redflag.xbox.vtx");
	
	AddFileToDownloadsTable("materials/models/mapmodels/flags/axisflag.vmt");
	AddFileToDownloadsTable("materials/models/mapmodels/flags/axisflag.vtf");
	AddFileToDownloadsTable("materials/models/mapmodels/flags/neutralflag.vmt");
	AddFileToDownloadsTable("materials/models/mapmodels/flags/neutralflag.vtf");
	AddFileToDownloadsTable("materials/models/extras/sctf/speech_info_blueflag.vmt");
	AddFileToDownloadsTable("materials/models/extras/sctf/speech_info_blueflag.vtf");
	AddFileToDownloadsTable("materials/models/extras/sctf/speech_info_redflag.vmt");
	AddFileToDownloadsTable("materials/models/extras/sctf/speech_info_redflag.vtf");
	
	AddFileToDownloadsTable("sound/smctf/ctf_win.mp3");
	AddFileToDownloadsTable("sound/smctf/ctf_lose.mp3");
	AddFileToDownloadsTable("sound/smctf/ctf_draw.mp3");
	
	PrecacheModel(MDL_LASER, true);
	PrecacheModel("models/mapmodels/pole.mdl", true);
	PrecacheModel("models/mapmodels/flags.mdl", true);
	PrecacheModel("models/extras/sctf/info_speech_blueflag.mdl", true);
	PrecacheModel("models/extras/sctf/info_speech_redflag.mdl", true);
	
	PrecacheSound(SOUND_FLAG_TAKEN, true);
	PrecacheSound(SOUND_FLAG_CAPTURE, true);
	PrecacheSound(SOUND_FLAG_DROP, true);
	PrecacheSound(SOUND_FLAG_LOST, true);
	PrecacheSound(SOUND_BLUE_FLAG_RETURN, true);
	PrecacheSound(SOUND_RED_FLAG_RETURN, true);
	PrecacheSound(SOUND_BLUE_FLAG_SCORE, true);
	PrecacheSound(SOUND_RED_FLAG_SCORE, true);
	PrecacheSound("weapons/explode3.wav", true);
	PrecacheSound("weapons/explode4.wav", true);
	PrecacheSound("weapons/explode5.wav", true);
	PrecacheSound(SOUND_TEAM_WIN, true);
	PrecacheSound(SOUND_TEAM_LOSE, true);
	PrecacheSound(SOUND_DRAW, true);
	
	g_iGlowSprite = PrecacheModel("sprites/blueglow2.vmt", true);
	g_iBeamSprite = PrecacheModel("materials/sprites/laser.vmt");
	g_iHaloSprite = PrecacheModel("materials/sprites/halo01.vmt");
	
	ParseFlagSpawns();
	ParsePlayerClasses();
	ParseRespawnWaveTimes();
	
	g_iTeamScore[RED_FLAG] = 0;
	g_iTeamScore[BLUE_FLAG] = 0;
	
	ServerCommand("sv_hudhint_sound 0");
	// This is set to the length of the winning sound
	ServerCommand("mp_chattime 18");
	ServerCommand("mp_playerid 2");
	ServerCommand("bot_chatter \"off\"");
	ServerCommand("bot_defer_to_human 1");
	ServerCommand("bot_join_after_player 0");
	ServerCommand("bot_quota_mode \"normal\"");
	ServerCommand("mp_ignore_round_win_conditions 1");
	
	g_hRemoveDroppedWeapons = CreateTimer(5.0, Timer_RemoveDroppedWeapons, _, TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
	
	// For showing players on radar
	g_iPlayerManager = FindEntityByClassname(0, "cs_player_manager");
	
	SDKHook(g_iPlayerManager, SDKHook_Think, Hook_OnPMThink);
	SDKHook(g_iPlayerManager, SDKHook_ThinkPost, Hook_OnPMThinkPost);
	
	// Enforce the timelimit
	g_hTimeLimitEnforcer = CreateTimer(10.0, Timer_CheckTimeLimit, GetTime(), TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
	
	// Prune x minutes old stats
	Stats_PruneOld();
	
	// Needed by the loghelper include to cache the teamnames.
	GetTeams();
}

public OnMapEnd()
{
	// General
	g_bIsIntermissionCalled = false;

	if(g_hStatusPanelUpdate != INVALID_HANDLE)
	{
		KillTimer(g_hStatusPanelUpdate);
		g_hStatusPanelUpdate = INVALID_HANDLE;
	}
	
	if(g_hRespawnWave[RED_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hRespawnWave[RED_FLAG]);
		g_hRespawnWave[RED_FLAG] = INVALID_HANDLE;
	}
	
	if(g_hRespawnWave[BLUE_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hRespawnWave[BLUE_FLAG]);
		g_hRespawnWave[BLUE_FLAG] = INVALID_HANDLE;
	}
	
	g_iRespawnWaveCountdownLength[RED_FLAG] = 0;
	g_iRespawnWaveTime[RED_FLAG] = 0;
	g_iRespawnWaveCountdownLength[BLUE_FLAG] = 0;
	g_iRespawnWaveTime[BLUE_FLAG] = 0;
	
	if(g_hRemoveDroppedWeapons != INVALID_HANDLE)
	{
		KillTimer(g_hRemoveDroppedWeapons);
		g_hRemoveDroppedWeapons = INVALID_HANDLE;
	}
	
	if(g_hTimeLimitEnforcer != INVALID_HANDLE)
	{
		KillTimer(g_hTimeLimitEnforcer);
		g_hTimeLimitEnforcer = INVALID_HANDLE;
	}
	
	// Flags
	if(g_hPlayFlagEffects != INVALID_HANDLE)
	{
		KillTimer(g_hPlayFlagEffects);
		g_hPlayFlagEffects = INVALID_HANDLE;
	}
	
	if(g_hBeamFollowTimer[RED_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hBeamFollowTimer[RED_FLAG]);
		g_hBeamFollowTimer[RED_FLAG] = INVALID_HANDLE;
	}
	
	if(g_hBeamFollowTimer[BLUE_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hBeamFollowTimer[BLUE_FLAG]);
		g_hBeamFollowTimer[BLUE_FLAG] = INVALID_HANDLE;
	}
	
	if(g_hResetDroppedFlag[RED_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlag[RED_FLAG]);
		g_hResetDroppedFlag[RED_FLAG] = INVALID_HANDLE;
	}
	
	if(g_hResetDroppedFlag[BLUE_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlag[BLUE_FLAG]);
		g_hResetDroppedFlag[BLUE_FLAG] = INVALID_HANDLE;
	}
	
	if(g_hResetDroppedFlagCountdown[RED_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlagCountdown[RED_FLAG]);
		g_hResetDroppedFlagCountdown[RED_FLAG] = INVALID_HANDLE;
	}
	
	if(g_hResetDroppedFlagCountdown[BLUE_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlagCountdown[BLUE_FLAG]);
		g_hResetDroppedFlagCountdown[BLUE_FLAG] = INVALID_HANDLE;
	}
	
	g_iFlagProtectionStartTime[RED_FLAG] = -1;
	if(g_hResetFlagProtection[RED_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hResetFlagProtection[RED_FLAG]);
		g_hResetFlagProtection[RED_FLAG] = INVALID_HANDLE;
	}
	
	g_iFlagProtectionStartTime[BLUE_FLAG] = -1;
	if(g_hResetFlagProtection[BLUE_FLAG] != INVALID_HANDLE)
	{
		KillTimer(g_hResetFlagProtection[BLUE_FLAG]);
		g_hResetFlagProtection[BLUE_FLAG] = INVALID_HANDLE;
	}
	
	// Classes
	if(g_hClassKV != INVALID_HANDLE)
	{
		CloseHandle(g_hClassKV);
		g_hClassKV = INVALID_HANDLE;
	}
	
	if(g_hClassMenuCT != INVALID_HANDLE)
	{
		CloseHandle(g_hClassMenuCT);
		g_hClassMenuCT = INVALID_HANDLE;
	}
	
	if(g_hClassMenuT != INVALID_HANDLE)
	{
		CloseHandle(g_hClassMenuT);
		g_hClassMenuT = INVALID_HANDLE;
	}
}

public OnConfigsExecuted()
{
	g_bUnlimitedMagazine = GetConVarBool(g_hCVUnlimitedMagazine);
	g_bNadeTrails = GetConVarBool(g_hCVNadeTrails);
}

public OnClientPutInServer(client)
{
	//SDKHook(client, SDKHook_WeaponDrop, Hook_OnWeaponDrop);
	
	SDKHook(client, SDKHook_PreThink, Hook_OnPlayerPreThink);
	SDKHook(client, SDKHook_PreThinkPost, Hook_OnPlayerThink);
	SDKHook(client, SDKHook_Think, Hook_OnPlayerThink);
	SDKHook(client, SDKHook_ThinkPost, Hook_OnPlayerThink);
	SDKHook(client, SDKHook_PostThink, Hook_OnPlayerPostThink);
	SDKHook(client, SDKHook_PostThinkPost, Hook_OnPlayerThink);
}

public OnClientAuthorized(client, const String:auth[])
{
	if(!IsFakeClient(client))
	{
		Stats_JoinInsert(client);
		Stats_Connect(client);
	}
}

public OnClientDisconnect(client)
{
	if(g_hResetSpawnProtection[client] != INVALID_HANDLE)
	{
		KillTimer(g_hResetSpawnProtection[client]);
		g_hResetSpawnProtection[client] = INVALID_HANDLE;
	}
	
	g_iSpecTarget[client] = -1;
	
	g_iPlayerGrenade[client][GRENADE_HE] = 0;
	g_iPlayerGrenade[client][GRENADE_FLASH] = 0;
	g_iPlayerGrenade[client][GRENADE_SMOKE] = 0;
	g_iPlayerClass[client] = -1;
	g_bFirstSpawn[client] = true;
	
	// Did the player carry a flag?
	new iCarriedFlagColor;
	if(g_iPlayerCarriesFlag[BLUE_FLAG] == client)
		iCarriedFlagColor = BLUE_FLAG;
	else if(g_iPlayerCarriesFlag[RED_FLAG] == client)
		iCarriedFlagColor = RED_FLAG;
	// Player doesn't carry any flag
	else
		return;
	
	ResetFlag(iCarriedFlagColor);
	
	if(iCarriedFlagColor == BLUE_FLAG)
		EmitSoundToAll(SOUND_BLUE_FLAG_RETURN);
	else
		EmitSoundToAll(SOUND_RED_FLAG_RETURN);
	
	PrintCenterTextAll("%s Flag returned! (Disconnected)", g_sFlagRead[iCarriedFlagColor]);
	CPrintToChatAllEx(client, "%s{teamcolor}%s{green} flag returned! ({olive}Disconnected{green})", PREFIX, g_sFlagRead[iCarriedFlagColor]);
	
	LogPlayerEvent(client, "triggered", "smctf_flag_dropped");
	Stats_Drop(client);
}

public OnClientDisconnect_Post(client)
{
	// This was the last human player? Reset the scores.
	if(GetConVarBool(g_hCVResetScores))
	{
		if(Client_GetCount(true, false) == 0)
		{
			g_iTeamScore[RED_FLAG] = 0;
			g_iTeamScore[BLUE_FLAG] = 0;
			Team_SetScore(CS_TEAM_T, 0);
			Team_SetScore(CS_TEAM_CT, 0);
		}
	}
}

public Action:Event_OnPlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	if(GetClientTeam(client) < CS_TEAM_T)
		return Plugin_Continue;
	
	// Restart the round, if it's the 2nd player spawning.
	if(g_bFirstSpawn[client])
	{
		new iPlayerCount;
		for(new i=1;i<=MaxClients;i++)
		{
			if(IsClientInGame(i) && GetClientTeam(i) >= CS_TEAM_T)
				iPlayerCount++;
		}
		if(iPlayerCount == 2)
			SetConVarInt(mp_restartgame, 1);
	}
	
	// Reset player's screen fade
	PerformFade(client, 1, {0,0,0,0}, FFADE_IN|FFADE_PURGE);
	g_iSpecTarget[client] = -1;
	
	new Handle:hBuffer = StartMessageOne("KeyHintText", client);
	BfWriteByte(hBuffer, 1);
	BfWriteString(hBuffer, "");
	EndMessage();
	
	Stats_Spawn(client);
	
	// Reset old spawnprotection
	if(g_hResetSpawnProtection[client] != INVALID_HANDLE)
	{
		KillTimer(g_hResetSpawnProtection[client]);
		g_hResetSpawnProtection[client] = INVALID_HANDLE;
	}
	
	CreateTimer(0.5, Timer_ApplyClassDelayed, client, TIMER_FLAG_NO_MAPCHANGE);
	
	// Apply spawnprotection
	new Float:fSpawnProtection = GetConVarFloat(g_hCVSpawnProtection);
	if(fSpawnProtection > 0.0)
	{
		SetEntProp(client, Prop_Data, "m_takedamage", 0);
		SetEntityRenderMode(client, RENDER_TRANSCOLOR);
		SetEntityRenderColor(client, 0, 200, 255, 190);
		g_hResetSpawnProtection[client] = CreateTimer(fSpawnProtection, Timer_ResetSpawnProtection, client, TIMER_FLAG_NO_MAPCHANGE);
	}
	
	g_bFirstSpawn[client] = false;
	
	return Plugin_Continue;
}

// Player Death event
public Action:Event_OnPlayerDeath(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	OnFlagDropped(client);
	
	if(g_hResetSpawnProtection[client] != INVALID_HANDLE)
	{
		KillTimer(g_hResetSpawnProtection[client]);
		g_hResetSpawnProtection[client] = INVALID_HANDLE;
	}
	
	// Reset spawnprotection
	SetEntProp(client, Prop_Data, "m_takedamage", 2);
	SetEntityRenderMode(client, RENDER_TRANSCOLOR);
	SetEntityRenderColor(client, 255, 255, 255, 255);
	
	Client_RemoveAllWeapons(client, "", true);
	
	Effect_DissolvePlayerRagDoll(client);
}

public Action:Event_OnRoundStart(Handle:event, const String:name[], bool:dontBroadcast)
{
	g_bRoundEnded = false;
	
	g_bFlagDropped[BLUE_FLAG] = false;
	g_bFlagDropped[RED_FLAG] = false;
	g_iPlayerCarriesFlag[BLUE_FLAG] = -1;
	g_iPlayerCarriesFlag[RED_FLAG] = -1;
	
	new maxent = GetMaxEntities(), String:eName[64];
	for (new i=MaxClients;i<maxent;i++)
	{
		if ( IsValidEdict(i) && IsValidEntity(i) )
		{
			// remove bombzones and hostages so no normal gameplay could end the round
			if ( GetEdictClassname(i, eName, sizeof(eName)) )
			{
				if(StrContains(eName, "hostage_entity") != -1 
				|| StrContains(eName, "func_bomb_target") != -1  
				|| StrContains(eName, "func_buyzone") != -1 
				|| StrContains(eName, "func_hostage_rescue") != -1 )
				{
					RemoveEdict(i);
				}
				// remove any ctf entities left over from last round
				else if((StrContains(eName, "prop_dynamic") != -1  
					 || StrContains(eName, "env_beam") != -1)  
					 && GetEntPropString(i, Prop_Data, "m_iName", eName, sizeof(eName))
					 && StrContains(eName, "sctf_") != -1)
				{
					AcceptEntityInput(i, "Kill");
				}
			}
		}
	}
	
	g_iFlagEntity[RED_FLAG] = -1;
	g_iFlagEntity[BLUE_FLAG] = -1;
	g_iPoleEntity[RED_FLAG] = -1;
	g_iPoleEntity[BLUE_FLAG] = -1;
	g_iDropBeam[RED_FLAG] = -1;
	g_iDropBeam[BLUE_FLAG] = -1;
	g_iSpeechIcon[RED_FLAG] = -1;
	g_iSpeechIcon[BLUE_FLAG] = -1;
	
	SpawnFlag(RED_FLAG);
	SpawnFlag(BLUE_FLAG);
	
	Team_SetScore(g_iColorTeam[BLUE_FLAG], g_iTeamScore[BLUE_FLAG]);
	Team_SetScore(g_iColorTeam[RED_FLAG], g_iTeamScore[RED_FLAG]);
	
	if(g_hStatusPanelUpdate == INVALID_HANDLE)
		g_hStatusPanelUpdate = CreateTimer(1.0, Timer_DrawStatusPanel, _, TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
	
	if(g_hRespawnWave[RED_FLAG] == INVALID_HANDLE)
	{
		g_iRespawnWaveTime[RED_FLAG] = GetTime();
		g_iRespawnWaveCountdownLength[RED_FLAG] = GetCurrentRespawnWaveTime();
		g_hRespawnWave[RED_FLAG] = CreateTimer(1.0, Timer_RespawnWave, RED_FLAG, TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
	}
	
	if(g_hRespawnWave[BLUE_FLAG] == INVALID_HANDLE)
	{
		g_iRespawnWaveTime[BLUE_FLAG] = GetTime();
		g_iRespawnWaveCountdownLength[BLUE_FLAG] = GetCurrentRespawnWaveTime();
		g_hRespawnWave[BLUE_FLAG] = CreateTimer(1.0, Timer_RespawnWave, BLUE_FLAG, TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
	}
	
	if(g_hPlayFlagEffects == INVALID_HANDLE)
		g_hPlayFlagEffects = CreateTimer(1.0, Timer_PlayFlagEffects, _, TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
}

public Action:Event_OnRoundEnd(Handle:event, const String:name[], bool:dontBroadcast)
{
	g_bRoundEnded = true;
	ResetFlag(BLUE_FLAG);
	ResetFlag(RED_FLAG);
	
	Team_SetScore(g_iColorTeam[BLUE_FLAG], g_iTeamScore[BLUE_FLAG]);
	Team_SetScore(g_iColorTeam[RED_FLAG], g_iTeamScore[RED_FLAG]);
}

/**
 * 
 * Timer Callbacks
 */ 

public Action:Timer_RespawnWave(Handle:timer, any:color)
{
	new iTimeRemaining;
	// Stop the Timer on round end
	if(g_bRoundEnded)
	{
		g_hRespawnWave[color] = INVALID_HANDLE;
		return Plugin_Stop;
	}
	
	iTimeRemaining = g_iRespawnWaveCountdownLength[color] - GetTime() + g_iRespawnWaveTime[color];
	if(iTimeRemaining <= 0)
	{
		g_iRespawnWaveCountdownLength[color] = GetCurrentRespawnWaveTime() + Math_GetRandomInt(-2, 2);
		g_iRespawnWaveTime[color] = GetTime();
	}
	
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i)
		&& !IsPlayerAlive(i)
		&& GetClientTeam(i) == g_iColorTeam[color])
		{
			Client_PrintKeyHintText(i, "Next Respawn: %ds", iTimeRemaining);
			if(iTimeRemaining <= 0)
				CS_RespawnPlayer(i);
		}
	}
	
	return Plugin_Continue;
}

public Action:Timer_RemoveDroppedWeapons(Handle:timer, any:data)
{
	new iMaxEntities = GetMaxEntities();
	decl String:sClassName[64];
	for(new i=MaxClients+1;i<iMaxEntities;i++)
	{
		if(IsValidEntity(i)
		&& IsValidEdict(i)
		&& GetEdictClassname(i, sClassName, sizeof(sClassName))
		&& StrContains(sClassName, "weapon_") != -1
		&& GetEntPropEnt(i, Prop_Data, "m_hOwnerEntity") == -1)
		{
			RemoveEdict(i);
		}
	}
	return Plugin_Continue;
}

public Action:Timer_PrintResultInfo(Handle:timer, any:data)
{
	// Draw
	if(data == 2)
	{
		PrintCenterTextAll("Draw!");
		CPrintToChatAll("%sDraw! No team has won the match! ({default}%d{green}-{default}%d{green})", PREFIX, g_iTeamScore[BLUE_FLAG], g_iTeamScore[RED_FLAG]);
	}
	// Blue team wins
	else if(data == BLUE_FLAG)
	{
		new String:sTeamColorTag[10];
		strtolower(g_sFlagRead[BLUE_FLAG], sTeamColorTag, sizeof(sTeamColorTag));
		Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
		PrintCenterTextAll("%s Team won!", g_sFlagRead[BLUE_FLAG]);
		CPrintToChatAll("%s%s%s{green} team has won the match! ({default}%d{green}-{default}%d{green})", PREFIX, sTeamColorTag, g_sFlagRead[BLUE_FLAG], g_iTeamScore[BLUE_FLAG], g_iTeamScore[RED_FLAG]);
	}
	// Red team wins
	else
	{
		new String:sTeamColorTag[10];
		strtolower(g_sFlagRead[RED_FLAG], sTeamColorTag, sizeof(sTeamColorTag));
		Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
		PrintCenterTextAll("%s Team won!", g_sFlagRead[RED_FLAG]);
		CPrintToChatAll("%s%s%s{green} team has won the match! ({default}%d{green}-{default}%d{green})", PREFIX, sTeamColorTag, g_sFlagRead[RED_FLAG], g_iTeamScore[RED_FLAG], g_iTeamScore[BLUE_FLAG]);
	}
	return Plugin_Stop;
}

// Force the timelimit set with mp_timelimit, even midround
public Action:Timer_CheckTimeLimit(Handle:timer, any:time)
{
	if(g_iTimeLimit != 0 && (GetTime() - time) >= g_iTimeLimit)
	{
		new iGameEnd  = FindEntityByClassname(-1, "game_end");
		if (iGameEnd == -1 && (iGameEnd = CreateEntityByName("game_end")) == -1) 
		{
			LogError("Unable to create entity \"game_end\"!");
		} 
		else 
		{
			g_hTimeLimitEnforcer = INVALID_HANDLE;
			AcceptEntityInput(iGameEnd, "EndGame");
			return Plugin_Stop;
		}
	}
	
	return Plugin_Continue;
}

public Action:Timer_ResetSpawnProtection(Handle:timer, any:client)
{
	g_hResetSpawnProtection[client] = INVALID_HANDLE;
	if(!IsClientInGame(client) || !IsPlayerAlive(client))
		return Plugin_Stop;
	
	SetEntProp(client, Prop_Data, "m_takedamage", 2);
	SetEntityRenderMode(client, RENDER_TRANSCOLOR);
	SetEntityRenderColor(client, 255, 255, 255, 255);
	
	return Plugin_Stop;
}

/**
 * 
 * ConVar Change Callbacks
 */ 
 
public ConVar_TimeLimitChanged(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if(!StrEqual(oldValue, newValue))
	{
		g_iTimeLimit = StringToInt(newValue)*60;
	}
}

public ConVar_OnChanged(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if(convar == g_hCVUnlimitedMagazine)
		g_bUnlimitedMagazine = bool:StringToInt(newValue);
	else if(convar == g_hCVNadeTrails)
		g_bNadeTrails = bool:StringToInt(newValue);
}

public ConVar_OnRestartGame(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if(StringToInt(newValue) > 0)
	{
		g_iTeamScore[RED_FLAG] = 0;
		g_iTeamScore[BLUE_FLAG] = 0;
	}
}

/**
 * 
 * Usermessage Callbacks
 */ 
 
public Action:_VGuiMenu(UserMsg:msg_id, Handle:bf, const players[], playersNum, bool:reliable, bool:init)
{
	if(g_bIsIntermissionCalled)
	{
		return;
	}

	new String:Type[10];
	BfReadString(bf, Type, sizeof(Type));

	if(strcmp(Type, "scores", false) == 0)
	{
		if(BfReadByte(bf) == 1 && BfReadByte(bf) == 0)
		{
			g_bIsIntermissionCalled = true;
			
			// Draw!
			if(g_iTeamScore[BLUE_FLAG] == g_iTeamScore[RED_FLAG])
			{
				EmitSoundToAll(SOUND_DRAW);
				
				Stats_Draw();
				
				CreateTimer(0.1, Timer_PrintResultInfo, 2, TIMER_FLAG_NO_MAPCHANGE);
			}
			// Blue team wins
			else if(g_iTeamScore[BLUE_FLAG] > g_iTeamScore[RED_FLAG])
			{
				EmitSoundToTeam(g_iColorTeam[BLUE_FLAG], SOUND_TEAM_WIN);
				EmitSoundToTeam(g_iColorTeam[RED_FLAG], SOUND_TEAM_LOSE);
				
				LogTeamEvent(g_iColorTeam[BLUE_FLAG], "triggered", "smctf_team_win_ct");
				LogTeamEvent(g_iColorTeam[RED_FLAG], "triggered", "smctf_team_lose_t");
				
				Stats_Win(g_iColorTeam[BLUE_FLAG]);
				Stats_Lose(g_iColorTeam[RED_FLAG]);
				
				CreateTimer(0.1, Timer_PrintResultInfo, BLUE_FLAG, TIMER_FLAG_NO_MAPCHANGE);
			}
			// Red team wins
			else
			{
				EmitSoundToTeam(g_iColorTeam[RED_FLAG], SOUND_TEAM_WIN);
				EmitSoundToTeam(g_iColorTeam[BLUE_FLAG], SOUND_TEAM_LOSE);
				
				LogTeamEvent(g_iColorTeam[RED_FLAG], "triggered", "smctf_team_win_t");
				LogTeamEvent(g_iColorTeam[BLUE_FLAG], "triggered", "smctf_team_lose_ct");
				
				Stats_Win(g_iColorTeam[RED_FLAG]);
				Stats_Lose(g_iColorTeam[BLUE_FLAG]);
				
				CreateTimer(0.1, Timer_PrintResultInfo, RED_FLAG, TIMER_FLAG_NO_MAPCHANGE);
			}
		}
	}
}


/**
 * SDKHooks
 */

public Action:CS_OnCSWeaponDrop(client, weapon)
//public Action:Hook_OnWeaponDrop(client, weapon)
{
	decl String:sWeapon[60];
	// Don't allow weapondrop.
	if(IsClientInGame(client)
	&& IsPlayerAlive(client) 
	&& !IsClientObserver(client)
	&& weapon > 0
	&& IsValidEntity(weapon) 
	&& IsValidEdict(weapon)
	&& GetEdictClassname(weapon, sWeapon, sizeof(sWeapon))
	// Allow dropping of grenades, as you won't be able to throw them otherwise :P
	&& !StrEqual(sWeapon, "weapon_hegrenade")
	&& !StrEqual(sWeapon, "weapon_flashbang")
	&& !StrEqual(sWeapon, "weapon_smokegrenade"))
	{
		//CPrintToChat(client, "%sYou're not allowed to drop your weapons!", PREFIX);
		return Plugin_Handled;
	}
	return Plugin_Continue;
}

public Hook_OnPlayerThink(client)
{
	Hook_OnPMThink(g_iPlayerManager);
}

public Hook_OnPlayerPreThink(client)
{
	Hook_OnPMThink(g_iPlayerManager);
	if(IsClientInGame(client) && (!IsPlayerAlive(client) || IsClientObserver(client)))
	{
		new Obs_Mode:iObserverMode = Client_GetObserverMode(client);
		new iSpecTarget = Client_GetObserverTarget(client);
		
		// Target got a flag and is in first or thirdperson spec mode
		if(iSpecTarget != -1
		&& (g_iPlayerCarriesFlag[RED_FLAG] == iSpecTarget
		|| g_iPlayerCarriesFlag[BLUE_FLAG] == iSpecTarget)
		&& (iObserverMode == OBS_MODE_IN_EYE 
		|| iObserverMode == OBS_MODE_CHASE))
		{
			// Didn't spectate him before 
			if(iSpecTarget != g_iSpecTarget[client])
			{
				new iFlagColor = RED_FLAG;
				if(g_iPlayerCarriesFlag[BLUE_FLAG] == iSpecTarget)
					iFlagColor = BLUE_FLAG;
				// Fade his screen either
				PerformFade(client, 255, g_iFlagRGBColor[iFlagColor], FFADE_OUT|FFADE_STAYOUT|FFADE_PURGE);
			}
			g_iSpecTarget[client] = iSpecTarget;
		}
		// Target doesn't carry a flag or wrong spec mode
		else
		{
			if(g_iSpecTarget[client] != -2)
			{
				PerformFade(client, 1, {0,0,0,0}, FFADE_IN|FFADE_PURGE);
			}
			// only unfade the screen once, don't spam;)
			g_iSpecTarget[client] = -2;
		}
	}
}

public Hook_OnPlayerPostThink(client)
{
	// Reset money
	SetEntProp(client, Prop_Send, "m_iAccount", 0);
	
	Hook_OnPMThink(g_iPlayerManager);
	
	// Unlimited ammo
	if(g_bUnlimitedMagazine && IsClientInGame(client) && IsPlayerAlive(client) && GetClientButtons(client) & IN_ATTACK)
	{
		new iCurrentWeapon = GetEntPropEnt(client, Prop_Send, "m_hActiveWeapon");
		decl String:sWeapon[64];
		if (iCurrentWeapon != -1
		&& IsValidEntity(iCurrentWeapon)
		&& IsValidEdict(iCurrentWeapon)
		&& GetEdictClassname(iCurrentWeapon, sWeapon, sizeof(sWeapon))
		&& StrContains(sWeapon, "weapon_", true) != -1
		&& GetEntProp(iCurrentWeapon, Prop_Send, "m_iClip1") == 0)
		{
			new iAmmoType = GetEntProp(iCurrentWeapon, Prop_Send, "m_iPrimaryAmmoType");
			if(iAmmoType > -1)
			{
				SetEntData(client, g_iAmmoOffset + iAmmoType*4, 200, 4, true); // HMM, just a random value. I like 90!
			}
		}
	}
	
	// Recreate the beam follow if standing still
	new iColor;
	if(g_iPlayerCarriesFlag[RED_FLAG] == client)
		iColor = RED_FLAG;
	else if(g_iPlayerCarriesFlag[BLUE_FLAG] == client)
		iColor = BLUE_FLAG;
	else
		return;
	
	if(!IsValidEntity(g_iSpeechIcon[iColor]) || !IsValidEdict(g_iSpeechIcon[iColor]))
		return;
	
	new Float:vecVelocity[3];
	vecVelocity[0] = GetEntPropFloat(client, Prop_Send, "m_vecVelocity[0]");
	vecVelocity[1] = GetEntPropFloat(client, Prop_Send, "m_vecVelocity[1]");
	vecVelocity[2] = GetEntPropFloat(client, Prop_Send, "m_vecVelocity[2]");
	if(vecVelocity[0] == 0.0 && vecVelocity[1] == 0.0 && vecVelocity[2] == 0.0 && g_hBeamFollowTimer[iColor] == INVALID_HANDLE)
	{
		g_hBeamFollowTimer[iColor] = CreateTimer(1.0, Timer_CreateBeamFollow, iColor, TIMER_FLAG_NO_MAPCHANGE);
	}
}

// Create grenade trails
public OnEntityCreated(entity, const String:classname[])
{
	if(!g_bNadeTrails)
		return;
	
	new color[4] = {0,...};
	if(StrContains(classname, "smokegrenade_projectile") != -1)
		color = {139, 137, 137, 255};
	else if(StrContains(classname, "hegrenade_projectile") != -1)
		color = {34, 139, 34, 255};
	else if(StrContains(classname, "flashbang_projectile") != -1)
		color = {255, 215, 0, 255};
		
	if(color[3] != 0)
	{
		TE_SetupBeamFollow(entity, g_iBeamSprite, g_iHaloSprite, 10.0, 1.0, 1.0, 1, color);
		TE_SendToAll();
	}
}

public Action:Command_SayHook(client, args)
{
	// Easy ctfstats chathook
	new String:text[512];
	GetCmdArgString(text, sizeof(text));
	StripQuotes(text);
	
	if(StrEqual(text[0], "ctfstats", false))
	{
		Command_Stats(client, 0);
		return Plugin_Handled;
	}
	else if(StrEqual(text[0], "class", false))
	{
		Command_ClassMenu(client, 0);
		return Plugin_Handled;
	}
	return Plugin_Continue;
}

stock strtolower(const String:source[], String:dest[], destLen)
{
	new len = strlen(source);
	for(new i=0;i<len;i++)
		Format(dest, destLen, "%s%c", dest, CharToLower(source[i]));
}

/**
 * Emits a sound to all members of a team.
 *
 * @param team			Team index to emit the sound to.
 * @param sample		Sound file name relative to the "sounds" folder.
 * @param entity		Entity to emit from.
 * @param channel		Channel to emit with.
 * @param level			Sound level.
 * @param flags			Sound flags.
 * @param volume		Sound volume.
 * @param pitch			Sound pitch.
 * @param speakerentity	Unknown.
 * @param origin		Sound origin.
 * @param dir			Sound direction.
 * @param updatePos		Unknown (updates positions?)
 * @param soundtime		Alternate time to play sound for.
 * @param ...			Optional list of Float[3] arrays to specify additional origins.
 * @noreturn
 * @error				Invalid client index.
 */
stock EmitSoundToTeam(const team,
				 const String:sample[],
				 entity = SOUND_FROM_PLAYER,
				 channel = SNDCHAN_AUTO,
				 level = SNDLEVEL_NORMAL,
				 flags = SND_NOFLAGS,
				 Float:volume = SNDVOL_NORMAL,
				 pitch = SNDPITCH_NORMAL,
				 speakerentity = -1,
				 const Float:origin[3] = NULL_VECTOR,
				 const Float:dir[3] = NULL_VECTOR,
				 bool:updatePos = true,
				 Float:soundtime = 0.0,
				 any:...)
{
	new clients[MaxClients];
	new total = 0;
	
	for (new i=1; i<=MaxClients; i++)
	{
		if (IsClientInGame(i) && GetClientTeam(i) == team)
		{
			clients[total++] = i;
		}
	}
	
	if (!total)
	{
		return;
	}
	
	EmitSound(clients, total, sample, entity, channel, 
		level, flags, volume, pitch, speakerentity,
		origin, dir, updatePos, soundtime);
}

stock PerformFade(client, duration, const color[4], flags)
{
	new Handle:hFadeClient = StartMessageOne("Fade", client);
	BfWriteShort(hFadeClient, duration);	// FIXED 16 bit, with SCREENFADE_FRACBITS fractional, seconds duration
	BfWriteShort(hFadeClient, 0);		// FIXED 16 bit, with SCREENFADE_FRACBITS fractional, seconds duration until reset (fade & hold)
	BfWriteShort(hFadeClient, flags); // fade type (in / out)
	BfWriteByte(hFadeClient, color[0]);	// fade red
	BfWriteByte(hFadeClient, color[1]);	// fade green
	BfWriteByte(hFadeClient, color[2]);	// fade blue
	BfWriteByte(hFadeClient, color[3]);	// fade alpha
	EndMessage();
}

// Stock by psychonic
// http://forums.alliedmods.net/showpost.php?p=1294224&postcount=2
stock MyAddServerTag(const String:tag[])
{
	decl String:currtags[128];
	if (sv_tags == INVALID_HANDLE)
	{
		return;
	}
	
	GetConVarString(sv_tags, currtags, sizeof(currtags));
	if (StrContains(currtags, tag) > -1)
	{
		// already have tag
		return;
	}
	
	decl String:newtags[128];
	Format(newtags, sizeof(newtags), "%s%s%s", currtags, (currtags[0]!=0)?",":"", tag);
	new flags = GetConVarFlags(sv_tags);
	SetConVarFlags(sv_tags, flags & ~FCVAR_NOTIFY);
	SetConVarString(sv_tags, newtags);
	SetConVarFlags(sv_tags, flags);
}

ParseRespawnWaveTimes()
{
	ClearArray(g_hRespawnWaveRules);
	
	decl String:sPath[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, sPath, sizeof(sPath), "configs/smctf_respawnwaves.cfg");
	
	if(!FileExists(sPath))
		SetFailState("Can't find respawn time config: %s", sPath);
	
	new Handle:hKV = CreateKeyValues("SCTFRespawnWaves");
	FileToKeyValues(hKV, sPath);
	
	if(!KvGotoFirstSubKey(hKV))
		SetFailState("Failed parsing respawn time config: %s", sPath);
	
	new iArray[2];
	do
	{
		iArray[0] = KvGetNum(hKV, "until_players");
		iArray[1] = KvGetNum(hKV, "respawn_time");
		if(iArray[0] < 1 || iArray[1] < 1)
			continue;
		PushArrayArray(g_hRespawnWaveRules, iArray, 2);
	}
	while(KvGotoNextKey(hKV));
	
	CloseHandle(hKV);
}

GetCurrentRespawnWaveTime()
{
	new iSize = GetArraySize(g_hRespawnWaveRules);
	new iArray[2], iUntilPlayers, iRespawnTime;
	
	// Get playing client count
	new iNumPlayers;
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i) && GetClientTeam(i) >= 2)
			iNumPlayers++;
	}
	
	for(new i=0;i<iSize;i++)
	{
		GetArrayArray(g_hRespawnWaveRules, i, iArray, 2);
		
		if(iNumPlayers <= iArray[0] && (iUntilPlayers == 0 || iArray[0] < iUntilPlayers))
		{
			iUntilPlayers = iArray[0];
			iRespawnTime = iArray[1];
		}
	}
	
	return iRespawnTime;
}