/*
 * Handles the player classes
 * Part of SM:CTF
 *
 * visit http://www.wcfan.de/
 */
#include <sourcemod>
#include <sdktools>
#include <cstrike>

#define GRENADE_HE 0
#define GRENADE_FLASH 1
#define GRENADE_SMOKE 2

new Handle:g_hClassKV = INVALID_HANDLE;
new Handle:g_hClassMenuCT = INVALID_HANDLE;
new Handle:g_hClassMenuT = INVALID_HANDLE;

new g_iPlayerClass[MAXPLAYERS+2] = {-1, ...};
new g_iPlayerGrenade[MAXPLAYERS+2][3];

new g_iTotalClasses = 0;
new bool:g_bUnlimitedMagazine = false;
new g_iAmmoOffset;

// Handles the /class menu selection and applies the chosen class
public Menu_ClassMenu(Handle:menu, MenuAction:action, param1, param2)
{
	
	if (action == MenuAction_Select) {
		decl String:info[35];
		GetMenuItem(menu, param2, info, sizeof(info));
		
		// This should never happen
		if(!KvJumpToKey(g_hClassKV, info))
		{
			PrintToChat(param1, "%sOoops! An error occured. Type /class to try again.", PREFIX);
			return;
		}
		
		// Cached here, since the next line overwrites the value
		new bool:bFirstClass = false;
		if(g_iPlayerClass[param1] == -1)
			bFirstClass = true;
		
		KvGetSectionSymbol(g_hClassKV, g_iPlayerClass[param1]);
		
		KvRewind(g_hClassKV);
		
		// Apply the class directly, if this is the first class the player chooses when alive.
		if(bFirstClass && IsPlayerAlive(param1))
			ApplyClass(param1);
		// Apply the class the next time he spawns
		else
		{
			decl String:sName[35], String:sDescription[128];
			KvJumpToKey(g_hClassKV, info);
			KvGetString(g_hClassKV, "name", sName, sizeof(sName));
			KvGetString(g_hClassKV, "description", sDescription, sizeof(sDescription));
			new iHealth = KvGetNum(g_hClassKV, "health");
			new Float:fSpeed = KvGetFloat(g_hClassKV, "speed");
			KvRewind(g_hClassKV);
			CPrintToChat(param1, "%sNext round, you'll be a {red}%s{green}, %s. Health: {olive}%d{green}, Speed: {olive}%d%{green}.", PREFIX, sName, sDescription, iHealth, RoundToFloor(fSpeed*100.0));
		}
	}
}

// Handles the /class command to show the menu
public Action:Command_ClassMenu(client, args)
{
	if(!client)
	{
		ReplyToCommand(client, "SM:CTF > You have to be ingame.");
		return Plugin_Handled;
	}
	
	ShowClassMenu(client);
	
	return Plugin_Handled;
}

// Gives grenades back, if the player still got some due to his class
public Action:Event_OnWeaponFire(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	decl String:sWeapon[32];
	GetEventString(event, "weapon", sWeapon, sizeof(sWeapon));
	
	if(StrEqual(sWeapon, "hegrenade"))
	{
		if(g_iPlayerGrenade[client][GRENADE_HE] != 0)
			g_iPlayerGrenade[client][GRENADE_HE]--;
		
		if(g_iPlayerGrenade[client][GRENADE_HE] > 0)
		{
			new iWeapon = GivePlayerItem(client, "weapon_hegrenade");
			if(iWeapon != INVALID_ENT_REFERENCE)
				Entity_SetParent(iWeapon, client);
			CPrintToChat(client, "%sYou've {lightgreen}%d{green} he grenade(s) left.", PREFIX, g_iPlayerGrenade[client][GRENADE_HE]);
		}
		/*else
		{
			// Has been the last grenade? Switch back to previous weapon
			CreateTimer(1.0, Timer_SwitchToLastWeapon, client, TIMER_FLAG_NO_MAPCHANGE);
		}*/
	}
	else if(StrEqual(sWeapon, "flashbang"))
	{
		if(g_iPlayerGrenade[client][GRENADE_FLASH] != 0)
			g_iPlayerGrenade[client][GRENADE_FLASH]--;
		
		if(g_iPlayerGrenade[client][GRENADE_FLASH] > 0)
		{
			new iWeapon = GivePlayerItem(client, "weapon_flashbang");
			if(iWeapon != INVALID_ENT_REFERENCE)
				Entity_SetParent(iWeapon, client);
			CPrintToChat(client, "%sYou've {lightgreen}%d{green} flashbang(s) left.", PREFIX, g_iPlayerGrenade[client][GRENADE_FLASH]);
		}
		/*else
		{
			// Has been the last grenade? Switch back to previous weapon
			CreateTimer(1.0, Timer_SwitchToLastWeapon, client, TIMER_FLAG_NO_MAPCHANGE);
		}*/
	}
	else if(StrEqual(sWeapon, "smokegrenade"))
	{
		if(g_iPlayerGrenade[client][GRENADE_SMOKE] != 0)
			g_iPlayerGrenade[client][GRENADE_SMOKE]--;
		
		if(g_iPlayerGrenade[client][GRENADE_SMOKE] > 0)
		{
			new iWeapon = GivePlayerItem(client, "weapon_smokegrenade");
			if(iWeapon != INVALID_ENT_REFERENCE)
				Entity_SetParent(iWeapon, client);
			CPrintToChat(client, "%sYou've {lightgreen}%d{green} smoke grenade(s) left.", PREFIX, g_iPlayerGrenade[client][GRENADE_SMOKE]);
		}
		/*else
		{
			// Has been the last grenade? Switch back to previous weapon
			CreateTimer(1.0, Timer_SwitchToLastWeapon, client, TIMER_FLAG_NO_MAPCHANGE);
		}*/
	}
}

// Timer to apply the class after spawn
// Needs to be delayed, since the speed could get reset right after spawn
public Action:Timer_ApplyClassDelayed(Handle:timer, any:client)
{
	if(IsClientInGame(client) && IsPlayerAlive(client))
		ApplyClass(client);
	
	return Plugin_Stop;
}

/*
public Action:Timer_SwitchToLastWeapon(Handle:timer, any:client)
{
	if(IsClientInGame(client) && IsPlayerAlive(client))
	{
		// Has been the last grenade? Switch back to previous weapon
		new iLastWeapon = GetEntPropEnt(client, Prop_Send, "m_hLastWeapon");
		decl String:sWeapon[32];
		if(iLastWeapon != -1)
		{
			GetEdictClassname(iLastWeapon, sWeapon, sizeof(sWeapon));
			FakeClientCommand(client, "use %s", sWeapon);
		}
	}
	
	return Plugin_Stop;
}*/

// Handles teamchanges and resets player's class
public Action:Event_OnPlayerTeam(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	g_iPlayerGrenade[client][GRENADE_HE] = 0;
	g_iPlayerGrenade[client][GRENADE_FLASH] = 0;
	g_iPlayerGrenade[client][GRENADE_SMOKE] = 0;
	g_iPlayerClass[client] = -1;

	// Changed midgame by a plugin?
	if(client && IsPlayerAlive(client))
		ApplyClass(client); // ApplyClass shows the menu, if there is no class to apply
}

// Removes the previous classes and reparses them out of the config file
ParsePlayerClasses()
{
	if(g_hClassKV != INVALID_HANDLE)
	{
		CloseHandle(g_hClassKV);
		g_hClassKV = INVALID_HANDLE;
	}
	
	if(g_hClassMenuCT != INVALID_HANDLE)
	{
		CloseHandle(g_hClassMenuCT);
		g_hClassMenuCT = INVALID_HANDLE;
	}
	
	if(g_hClassMenuT != INVALID_HANDLE)
	{
		CloseHandle(g_hClassMenuT);
		g_hClassMenuT = INVALID_HANDLE;
	}
	
	// Reset all player info
	for(new i=1;i<=MaxClients;i++)
	{
		g_iPlayerGrenade[i][GRENADE_HE] = 0;
		g_iPlayerGrenade[i][GRENADE_FLASH] = 0;
		g_iPlayerGrenade[i][GRENADE_SMOKE] = 0;
		g_iPlayerClass[i] = -1;
	}
	
	g_iTotalClasses = 0;
	
	g_hClassKV = CreateKeyValues("Classes");
	new String:sFile[256];
	BuildPath(Path_SM, sFile, sizeof(sFile), "configs/smctf_classes.cfg");
	FileToKeyValues(g_hClassKV, sFile);
	
	if (!KvGotoFirstSubKey(g_hClassKV))
	{
		CloseHandle(g_hClassKV);
		return;
	}
	
	// Need 2 menus, due to the team only classes
	g_hClassMenuT = CreateMenu(Menu_ClassMenu);
	SetMenuTitle(g_hClassMenuT, "SM:CTF Player Classes");
	SetMenuExitBackButton(g_hClassMenuT, true);
	
	g_hClassMenuCT = CreateMenu(Menu_ClassMenu);
	SetMenuTitle(g_hClassMenuCT, "SM:CTF Player Classes");
	SetMenuExitBackButton(g_hClassMenuCT, true);
	
	decl String:sSectionName[35], String:sClassName[256], String:sDescription[128];
	new iTeam;
	do
	{
		KvGetSectionName(g_hClassKV, sSectionName, sizeof(sSectionName));
		KvGetString(g_hClassKV, "name", sClassName, sizeof(sClassName), "%/&|");
		KvGetString(g_hClassKV, "description", sDescription, sizeof(sDescription));
		if(!StrEqual(sClassName, "%/&|"))
		{
			Format(sClassName, sizeof(sClassName), "%s (%s)", sClassName, sDescription);
			
			// Add the class to both team's menus, if the key is not set
			iTeam = KvGetNum(g_hClassKV, "team", 0);
			if(!iTeam || iTeam == CS_TEAM_T)
			{
				AddMenuItem(g_hClassMenuT, sSectionName, sClassName);
			}
			if(!iTeam || iTeam == CS_TEAM_CT)
			{
				AddMenuItem(g_hClassMenuCT, sSectionName, sClassName);
			}
			
			// Precache the model, if set
			decl String:sModel[64];
			KvGetString(g_hClassKV, "model", sModel, sizeof(sModel), "xxx");
			if(!StrEqual(sModel, "xxx"))
			{
				Format(sModel, sizeof(sModel), "models/%s", sModel);
				PrecacheModel(sModel);
			}
			g_iTotalClasses++;
		}
		
	} while (KvGotoNextKey(g_hClassKV));

	KvRewind(g_hClassKV);
}

// Shows the right menu according to the player's team
ShowClassMenu(client)
{
	new iTeam = GetClientTeam(client);
	if(iTeam == CS_TEAM_CT)
		DisplayMenu(g_hClassMenuCT, client, MENU_TIME_FOREVER);
	else if(iTeam == CS_TEAM_T)
		DisplayMenu(g_hClassMenuT, client, MENU_TIME_FOREVER);
}


// Applies a class on a client
ApplyClass(client)
{
	g_iPlayerGrenade[client][GRENADE_HE] = 0;
	g_iPlayerGrenade[client][GRENADE_FLASH] = 0;
	g_iPlayerGrenade[client][GRENADE_SMOKE] = 0;
	
	// Bots always have a class autoassigned
	// Show classmenu to player, if he got no class or an invalid one
	if(!IsFakeClient(client) && (g_iPlayerClass[client] == -1 || !KvJumpToKeySymbol(g_hClassKV, g_iPlayerClass[client])))
	{
		// smlib function to strip the player
		// we're going to give them new weapons according to their class
		Client_RemoveAllWeapons(client, "", true);
		g_iPlayerClass[client] = -1;
		ShowClassMenu(client);
		return;
	}
	
	// Autoassign a random class to the bots
	if(IsFakeClient(client))
	{
		new iRandomClass = Math_GetRandomInt(0, g_iTotalClasses-1);
		new i = 0;
		KvGotoFirstSubKey(g_hClassKV);
		do
		{
			if(i == iRandomClass)
			{
				KvGetSectionSymbol(g_hClassKV, g_iPlayerClass[client]);
				break;
			}
			i++;
		} while (KvGotoNextKey(g_hClassKV));
	}
	
	// Set the health
	new iHealth = KvGetNum(g_hClassKV, "health", 0);
	if(iHealth > 0)
		SetEntityHealth(client, iHealth);
	
	// Set the speed
	// Using lmv method until the proper maxspeed method is fixed (15.02.2011)
	new Float:fSpeed = KvGetFloat(g_hClassKV, "speed", 0.0);
	if(fSpeed > 0.0)
		SetEntPropFloat(client, Prop_Send, "m_flLaggedMovementValue", fSpeed);
	
	// Set the model, if given for that class
	decl String:sModel[64];
	KvGetString(g_hClassKV, "model", sModel, sizeof(sModel), "xxx");
	if(!StrEqual(sModel, "xxx"))
	{
		Format(sModel, sizeof(sModel), "models/%s", sModel);
		SetEntityModel(client, sModel);
	}
	
	// Strip the client's weapons
	Client_RemoveAllWeapons(client, "", true);
	
	// Give him the weapons for the class he's in
	decl String:sWeapons[512], String:sWeaponAndAmmo[2][30], String:sWeapon[20][30], String:sWeaponPrefixed[30];
	KvGetString(g_hClassKV, "weapons", sWeapons, sizeof(sWeapons), "xxx");
	if(!StrEqual(sWeapons, "xxx"))
	{
		new iCount = ExplodeString(sWeapons, ",", sWeapon, 20, 30);
		new iWeapon, iAmmoType;
		for(new i=0;i<iCount;i++)
		{
			ExplodeString(sWeapon[i], ":", sWeaponAndAmmo, 2, 30);
			Format(sWeaponPrefixed, sizeof(sWeaponPrefixed), "weapon_%s", sWeaponAndAmmo[0]);
			// Give weapon
			iWeapon = GivePlayerItem(client, sWeaponPrefixed);
			if(iWeapon != -1)
			{
				Entity_SetParent(iWeapon, client);
				// Set Ammo
				if(strlen(sWeaponAndAmmo[1]) > 0)
				{
					if(StrEqual(sWeaponAndAmmo[0], "hegrenade"))
					{
						g_iPlayerGrenade[client][GRENADE_HE] = StringToInt(sWeaponAndAmmo[1]);
					}
					else if(StrEqual(sWeaponAndAmmo[0], "flashbang"))
					{
						g_iPlayerGrenade[client][GRENADE_FLASH] = StringToInt(sWeaponAndAmmo[1]);
					}
					else if(StrEqual(sWeaponAndAmmo[0], "smokegrenade"))
					{
						g_iPlayerGrenade[client][GRENADE_SMOKE] = StringToInt(sWeaponAndAmmo[1]);
					}
					// Usually not used anymore due to unlimited ammo
					else
					{
						iAmmoType = GetEntProp(iWeapon, Prop_Send, "m_iPrimaryAmmoType");
						if(iAmmoType != -1)
						{
							SetEntData(client, FindSendPropOffs("CCSPlayer", "m_iAmmo")+(iAmmoType*4), StringToInt(sWeaponAndAmmo[1]));
						}
					}
					strcopy(sWeaponAndAmmo[1], 30, "");
				}
			}
		}
	}
	
	decl String:sName[64], String:sDescription[256];
	KvGetString(g_hClassKV, "name", sName, sizeof(sName), "xxx");
	KvGetString(g_hClassKV, "description", sDescription, sizeof(sDescription), "xxx");
	CPrintToChat(client, "%sYou're a {red}%s{green}, %s. Health: {olive}%d{green}, Speed: {olive}%d%{green}.", PREFIX, sName, sDescription, iHealth, RoundToFloor(fSpeed*100.0));
	
	KvRewind(g_hClassKV);
	
	CPrintToChat(client, "%sType \"{blue}/class{green}\" to change your class.", PREFIX);
}