/*
 * Handles the flag actions
 * Part of SM:CTF
 *
 * visit http://www.wcfan.de/
 */
#include <sourcemod>

#define MDL_LASER "sprites/laser.vmt"

// Soundfiles
// Happen enemy take our flag
// Team listen that sound: Flag Color Team
#define SOUND_FLAG_TAKEN "doors/vent_open2.wav"
// Happen when a enemy cath the enemy flag
// Team listen that sound: Enemy of that flag
#define SOUND_FLAG_CAPTURE "items/battery_pickup.wav"
// Happen when a enemy drop the enemy flag
// Team listen that sound: Flag Color Team
#define SOUND_FLAG_DROP "buttons/blip1.wav"
// Happen when the team drop/lost enemy flag
// Team listen that sound: Enemy of that flag
#define SOUND_FLAG_LOST "ambient/alarms/warningbell1.wav"
// Happen when Blue Flag Return to base
// Team listen that sound: All Players
#define SOUND_BLUE_FLAG_RETURN "ambient/explosions/explode_7.wav"
#define SOUND_RED_FLAG_RETURN "ambient/explosions/explode_9.wav"
// Happen when some team score
// Team listen that sound: All Players
#define SOUND_BLUE_FLAG_SCORE "plats/elevbell1.wav"
#define SOUND_RED_FLAG_SCORE "ambient/alarms/klaxon1.wav"
// Happen when a team Wins the game
// Team listen that sound: Winners
#define SOUND_TEAM_WIN "smctf/ctf_win.mp3"
// Happen when a team Lose the game
// Team listen that sound: Losers
#define SOUND_TEAM_LOSE "smctf/ctf_lose.mp3"
// Happen when game end in Draw
// Team listen that sound: All Players
#define SOUND_DRAW "smctf/ctf_draw.mp3"

#define BLUE_FLAG 0
#define RED_FLAG 1

new Float:g_fFlagSpawnPoint[2][3];
new String:g_sFlagRead[][] = {"Blue", "Red"};
new g_iColorTeam[2] = {CS_TEAM_CT, CS_TEAM_T};

// Player, who carries the flag
new g_iPlayerCarriesFlag[2] = {-1, -1};
// Entity index of speech icon over flag carriers head
new g_iSpeechIcon[2] = {-1, -1};
// Timer to recreate the follow beam
new Handle:g_hBeamFollowTimer[2] = {INVALID_HANDLE, INVALID_HANDLE};
// Entity index of the flag
new g_iFlagEntity[2] = {-1, -1};
// Entity index of the pole
new g_iPoleEntity[2] = {-1, -1};
// Color of the flag carriers screen
new g_iFlagRGBColor[2][4] = {{0,0,255,35}, {255,0,0,35}};
// Is the flag dropped somewhere?
new bool:g_bFlagDropped[2] = {false, false};
// Timer to reset the dropped flag
new Handle:g_hResetDroppedFlag[2] = {INVALID_HANDLE, INVALID_HANDLE};
new Handle:g_hResetDroppedFlagCountdown[2] = {INVALID_HANDLE, INVALID_HANDLE};
new g_iResetDroppedFlagCountdown[2] = {0, 0};
new Float:g_fFlagDropPoint[2][3];
// Entity index of beam for dropped flag (used for touch events)
new g_iDropBeam[2] = {-1, -1};
// Timer to reset the flag's spawnprotection
new Handle:g_hResetFlagProtection[2] = {INVALID_HANDLE, INVALID_HANDLE};
new g_iFlagProtectionStartTime[2] = {-1, -1};

// Store, which player one spectates
new g_iSpecTarget[MAXPLAYERS+2] = {-2,...};

new g_iPlayerManager;
new g_iRadarOffset;
new g_iPlayerVIPOffset;
#if defined USE_HIDDENBOTS
new g_ibConnectedOffset;
new g_ibAliveOffset;
#endif

new g_iVIPShowSwitch;

// Team scores
new g_iTeamScore[2] = {0, 0};

new Handle:g_hPlayFlagEffects = INVALID_HANDLE;
new g_iGlowSprite = -1;
new g_iBeamSprite = -1;
new g_iHaloSprite = -1;

// Status hint update timer
new Handle:g_hStatusPanelUpdate = INVALID_HANDLE;

public EntOut_OnTouchFlag(const String:output[], caller, activator, Float:delay)
{
	AcceptEntityInput(caller, "TurnOff");
	AcceptEntityInput(caller, "TurnOn");
	
	// Ignore bots and dead players
	if(!IsPlayerAlive(activator)
	|| IsFakeClient(activator))
	{
		return;
	}
	
	new String:sName[35];
	GetEntPropString(caller, Prop_Data, "m_iName", sName, sizeof(sName));
	
	new iFlagColor;
	new iTeam = GetClientTeam(activator);
	new iTeamColor = g_iColorTeam[RED_FLAG] == iTeam?RED_FLAG:BLUE_FLAG;
	if(StrContains(sName, "red", false) != -1)
		iFlagColor = RED_FLAG;
	else
		iFlagColor = BLUE_FLAG;
	
	if(g_iColorTeam[iFlagColor] == iTeam && !GetConVarBool(g_hCVCaptureOwnFlag))
	{
		Client_PrintKeyHintText(activator, "You can't capture your own flag.");
		return;
	}
	
	// Make sure he isn't carrying a flag already
	if(g_iPlayerCarriesFlag[BLUE_FLAG] == activator
	|| g_iPlayerCarriesFlag[RED_FLAG] == activator)
	{
		Client_PrintKeyHintText(activator, "You're only able to carry one flag at a time.");
		return;
	}
	
	// Flag is proteced?
	if(g_hResetFlagProtection[iFlagColor] != INVALID_HANDLE)
	{
		Client_PrintKeyHintText(activator, "This flag is protected.\nWait %d seconds until you pick it up.", (GetConVarInt(g_hCVFlagProtectionTime) - GetTime() + g_iFlagProtectionStartTime[iFlagColor]));
		return;
	}
	
	Client_PrintKeyHintText(activator, "");
	
	// Touching player got the flag!
	g_iPlayerCarriesFlag[iFlagColor] = activator;
	g_bFlagDropped[iFlagColor] = false;
	
	// Return the flag immediately
	if(GetConVarBool(g_hCVReturnOnOwnTeam) && g_iColorTeam[iFlagColor] == iTeam)
	{
		EntOut_OnTouchPole("OnTouchedByEntity", g_iPoleEntity[iFlagColor], activator, 0.0);
		return;
	}
	
	// Hide the flag
	SetEntityRenderMode(g_iFlagEntity[iFlagColor], RENDER_TRANSCOLOR);
	SetEntityRenderColor(g_iFlagEntity[iFlagColor], 0, 0, 0, 0);
	
	// Kill the beam
	AcceptEntityInput(caller, "Kill");
	g_iDropBeam[iFlagColor] = -1;
	
	// Color the players's screen
	PerformFade(activator, 255, g_iFlagRGBColor[iFlagColor], FFADE_OUT|FFADE_STAYOUT|FFADE_PURGE);
	
	g_iResetDroppedFlagCountdown[iFlagColor] = 0;
	
	if(g_hResetDroppedFlag[iFlagColor] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlag[iFlagColor]);
		g_hResetDroppedFlag[iFlagColor] = INVALID_HANDLE;
	}
	
	if(g_hResetDroppedFlagCountdown[iFlagColor] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlagCountdown[iFlagColor]);
		g_hResetDroppedFlagCountdown[iFlagColor] = INVALID_HANDLE;
	}
	
	// Red player got the red flag back
	if(g_iColorTeam[iFlagColor] == CS_TEAM_T)
	{
		EmitSoundToTeam(CS_TEAM_T, SOUND_FLAG_CAPTURE);
		EmitSoundToTeam(CS_TEAM_CT, SOUND_FLAG_TAKEN);
	}
	// Blue player got the blue flag back
	else
	{
		EmitSoundToTeam(CS_TEAM_CT, SOUND_FLAG_CAPTURE);
		EmitSoundToTeam(CS_TEAM_T, SOUND_FLAG_TAKEN);
	}
	
	new String:sTeamColorTag[10];
	strtolower(g_sFlagRead[iTeamColor], sTeamColorTag, sizeof(sTeamColorTag));
	Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
	PrintCenterTextAll("%s Team has the %s Flag! (%N)", g_sFlagRead[iTeamColor], g_sFlagRead[iFlagColor], activator);
	CPrintToChatAll("%s%s%s{green} team has the {olive}%s{green} flag! ({default}%N{green})", PREFIX, sTeamColorTag, g_sFlagRead[iTeamColor], g_sFlagRead[iFlagColor], activator);
	
	SetEntProp(activator, Prop_Data, "m_iFrags", GetClientFrags(activator)+GetConVarInt(g_hCVScoreForTakeFlag));
	CPrintToChat(activator, "%sYou won{olive} %d{green} point for taking a flag!", PREFIX, GetConVarInt(g_hCVScoreForTakeFlag));
	
	LogPlayerEvent(activator, "triggered", "smctf_flag_captured");
	Stats_Capture(activator);
	
	CreateSpeechIcon(activator);
}

public EntOut_OnTouchPole(const String:output[], caller, activator, Float:delay)
{
	AcceptEntityInput(caller, "TurnOff");
	AcceptEntityInput(caller, "TurnOn");
	
	if(g_bRoundEnded || !IsPlayerAlive(activator) || IsFakeClient(activator))
	{
		return;
	}
	
	new String:sName[35];
	GetEntPropString(caller, Prop_Data, "m_iName", sName, sizeof(sName));
	
	// What color has this pole?
	new iPoleColor;
	new iTeam = GetClientTeam(activator);
	new iTeamColor = g_iColorTeam[RED_FLAG] == iTeam?RED_FLAG:BLUE_FLAG;
	if(StrContains(sName, "red", false) != -1)
		iPoleColor = RED_FLAG;
	else
		iPoleColor = BLUE_FLAG;
	
	// What color has the flag, the player carries?
	new iCarriedFlagColor;
	if(g_iPlayerCarriesFlag[BLUE_FLAG] == activator)
		iCarriedFlagColor = BLUE_FLAG;
	else if(g_iPlayerCarriesFlag[RED_FLAG] == activator)
		iCarriedFlagColor = RED_FLAG;
	// Player doesn't carry any flag
	else
	{
		// Flag is at home and enemy touches it
		if(g_iPlayerCarriesFlag[iPoleColor] == -1
		&& !g_bFlagDropped[iPoleColor]
		&& iTeam != g_iColorTeam[iPoleColor])
		{
			// Flag is proteced?
			if(g_hResetFlagProtection[iPoleColor] != INVALID_HANDLE)
			{
				Client_PrintKeyHintText(activator, "This flag is protected.\nWait %d seconds until you pick it up.", (GetConVarInt(g_hCVFlagProtectionTime) - GetTime() + g_iFlagProtectionStartTime[iPoleColor]));
				return;
			}
			
			// Remove the panel
			Client_PrintKeyHintText(activator, "");
			
			// Hide the flag
			SetEntityRenderMode(g_iFlagEntity[iPoleColor], RENDER_TRANSCOLOR);
			SetEntityRenderColor(g_iFlagEntity[iPoleColor], 0, 0, 0, 0);
			
			// Show the pole
			SetEntityRenderMode(g_iPoleEntity[iPoleColor], RENDER_TRANSCOLOR);
			SetEntityRenderColor(g_iPoleEntity[iPoleColor], 255, 255, 255, 255);
			
			// Touching player got the flag!
			g_iPlayerCarriesFlag[iPoleColor] = activator;
			
			// Color the players's screen
			PerformFade(activator, 255, g_iFlagRGBColor[iPoleColor], FFADE_OUT|FFADE_STAYOUT|FFADE_PURGE);
			
			// Red player got the red flag back
			if(g_iColorTeam[iPoleColor] == CS_TEAM_T)
			{
				EmitSoundToTeam(CS_TEAM_T, SOUND_FLAG_CAPTURE);
				EmitSoundToTeam(CS_TEAM_CT, SOUND_FLAG_TAKEN);
			}
			// Blue player got the blue flag back
			else
			{
				EmitSoundToTeam(CS_TEAM_CT, SOUND_FLAG_CAPTURE);
				EmitSoundToTeam(CS_TEAM_T, SOUND_FLAG_TAKEN);
			}
			
			new String:sTeamColorTag[10];
			strtolower(g_sFlagRead[iTeamColor], sTeamColorTag, sizeof(sTeamColorTag));
			Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
			PrintCenterTextAll("%s Team has the %s Flag! (%N)", g_sFlagRead[iTeamColor], g_sFlagRead[iPoleColor], activator);
			CPrintToChatAll("%s%s%s{green} team has the {olive}%s{green} flag! ({default}%N{green})", PREFIX, sTeamColorTag, g_sFlagRead[iTeamColor], g_sFlagRead[iPoleColor], activator);
			
			SetEntProp(activator, Prop_Data, "m_iFrags", GetClientFrags(activator)+GetConVarInt(g_hCVScoreForTakeFlag));
			CPrintToChat(activator, "%sYou won{olive} %d{green} point for taking a flag!", PREFIX, GetConVarInt(g_hCVScoreForTakeFlag));
			
			LogPlayerEvent(activator, "triggered", "smctf_flag_captured");
			Stats_Capture(activator);
			
			CreateSpeechIcon(activator);
		}
		return;
	}
	
	// Player brought the flag back!
	if(iPoleColor == iCarriedFlagColor
	&& iTeam == g_iColorTeam[iPoleColor])
	{
		if(iPoleColor == BLUE_FLAG)
			EmitSoundToAll(SOUND_BLUE_FLAG_RETURN);
		else
			EmitSoundToAll(SOUND_RED_FLAG_RETURN);
		
		new String:sTeamColorTag[10];
		strtolower(g_sFlagRead[iPoleColor], sTeamColorTag, sizeof(sTeamColorTag));
		Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
		PrintCenterTextAll("%s Flag returned! (%N)", g_sFlagRead[iPoleColor], activator);
		CPrintToChatAll("%s%s%s{green} flag returned! ({default}%N{green})", PREFIX, sTeamColorTag, g_sFlagRead[iPoleColor], activator);
		
		SetEntProp(activator, Prop_Data, "m_iFrags", GetClientFrags(activator)+GetConVarInt(g_hCVScoreForReturnFlag));
		CPrintToChat(activator, "%sYou won{olive} %d{green} point for bringing a flag back to your base!", PREFIX, GetConVarInt(g_hCVScoreForReturnFlag));
		
		ResetFlag(iPoleColor);
		
		// Reset player's screen fade
		PerformFade(activator, 1, {0,0,0,0}, FFADE_IN|FFADE_PURGE);
		
		LogPlayerEvent(activator, "triggered", "smctf_flag_returned");
		Stats_Return(activator);
	}
	// Player brought the enemy flag and ours is here!
	else if(iPoleColor != iCarriedFlagColor
		 && iTeam == g_iColorTeam[iPoleColor]
		 && !GetConVarBool(g_hCVRequireOwnFlagHome)
		 || (g_iPlayerCarriesFlag[iPoleColor] == -1
		 && !g_bFlagDropped[iPoleColor]))
	{
		// Flag is proteced?
		if(g_hResetFlagProtection[iPoleColor] != INVALID_HANDLE && GetConVarBool(g_hCVRequireOwnFlagHome))
		{
			Client_PrintKeyHintText(activator, "This flag is protected.\nWait %d seconds until you're able to score.", GetConVarInt(g_hCVFlagProtectionTime));
			return;
		}
		
		// Remove the panel
		Client_PrintKeyHintText(activator, "");
		
		if(iPoleColor == BLUE_FLAG)
			EmitSoundToAll(SOUND_BLUE_FLAG_SCORE);
		else
			EmitSoundToAll(SOUND_RED_FLAG_SCORE);
		
		new String:sTeamColorTag[10];
		strtolower(g_sFlagRead[iPoleColor], sTeamColorTag, sizeof(sTeamColorTag));
		Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
		PrintCenterTextAll("%s Team Score! (%N)", g_sFlagRead[iPoleColor], activator);
		CPrintToChatAll("%s%s%s{green} team scored! ({default}%N{green})", PREFIX, sTeamColorTag, g_sFlagRead[iPoleColor], activator);
		
		EmitSoundToClient(activator, "weapons/explode3.wav");
		EmitSoundToClient(activator, "weapons/explode4.wav");
		EmitSoundToClient(activator, "weapons/explode5.wav");
		
		SetEntProp(activator, Prop_Data, "m_iFrags", GetClientFrags(activator)+GetConVarInt(g_hCVScoreForScoreFlag));
		CPrintToChat(activator, "%sYou won{olive} %d{green} points for scoring!", PREFIX, GetConVarInt(g_hCVScoreForScoreFlag));
		
		ResetFlag(iCarriedFlagColor);
		
		// Reset player's screen fade
		PerformFade(activator, 1, {0,0,0,0}, FFADE_IN|FFADE_PURGE);
		
		// Add 1 point to the team score
		g_iTeamScore[iPoleColor]++;
		Team_SetScore(g_iColorTeam[iPoleColor], g_iTeamScore[iPoleColor]);
		
		LogPlayerEvent(activator, "triggered", "smctf_scored");
		Stats_Score(activator);
		
		// Team won!
		if(g_iTeamScore[iPoleColor] >= GetConVarInt(g_hCVScoreLimit))
		{
			new iGameEnd  = FindEntityByClassname(-1, "game_end");
			if (iGameEnd == -1 && (iGameEnd = CreateEntityByName("game_end")) == -1) 
			{
				LogError("Unable to create entity \"game_end\"!");
			} 
			else 
			{
				// The intermission is catched. No need to handle the winning scores here.
				AcceptEntityInput(iGameEnd, "EndGame");
			}
		}
	}
}

OnFlagDropped(client)
{
	// Did the player carry a flag?
	new iCarriedFlagColor;
	if(g_iPlayerCarriesFlag[BLUE_FLAG] == client)
		iCarriedFlagColor = BLUE_FLAG;
	else if(g_iPlayerCarriesFlag[RED_FLAG] == client)
		iCarriedFlagColor = RED_FLAG;
	// Player didn't carry any flag
	else
		return;
	
	g_bFlagDropped[iCarriedFlagColor] = true;
	g_iPlayerCarriesFlag[iCarriedFlagColor] = -1;
	
	// Remove speech icon
	if(IsValidEntity(g_iSpeechIcon[iCarriedFlagColor]))
	{
		AcceptEntityInput(g_iSpeechIcon[iCarriedFlagColor], "Kill");
		g_iSpeechIcon[iCarriedFlagColor] = -1;
	}
	
	g_iResetDroppedFlagCountdown[iCarriedFlagColor] = 0;
	
	if(g_hResetDroppedFlag[iCarriedFlagColor] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlag[iCarriedFlagColor]);
		g_hResetDroppedFlag[iCarriedFlagColor] = INVALID_HANDLE;
	}
	
	if(g_hResetDroppedFlagCountdown[iCarriedFlagColor] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlagCountdown[iCarriedFlagColor]);
		g_hResetDroppedFlagCountdown[iCarriedFlagColor] = INVALID_HANDLE;
	}
	
	if(g_hBeamFollowTimer[iCarriedFlagColor] != INVALID_HANDLE)
	{
		KillTimer(g_hBeamFollowTimer[iCarriedFlagColor]);
		g_hBeamFollowTimer[iCarriedFlagColor] = INVALID_HANDLE;
	}
	
	// CT lost the T flag
	if(g_iColorTeam[iCarriedFlagColor] == CS_TEAM_T)
	{
		EmitSoundToTeam(CS_TEAM_T, SOUND_FLAG_DROP);
		EmitSoundToTeam(CS_TEAM_CT, SOUND_FLAG_LOST);
	}
	else
	{
		EmitSoundToTeam(CS_TEAM_CT, SOUND_FLAG_DROP);
		EmitSoundToTeam(CS_TEAM_T, SOUND_FLAG_LOST);
	}
	
	// Get place name
	decl String:sLastPlaceName[64];
	GetEntPropString(client, Prop_Send, "m_szLastPlaceName", sLastPlaceName, sizeof(sLastPlaceName));
	if(strlen(sLastPlaceName) > 0)
		Format(sLastPlaceName, sizeof(sLastPlaceName), " {default}@{olive} %s{green}", sLastPlaceName);
	else
		Format(sLastPlaceName, sizeof(sLastPlaceName), "");
	
	new String:sTeamColorTag[10];
	strtolower(g_sFlagRead[iCarriedFlagColor], sTeamColorTag, sizeof(sTeamColorTag));
	Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
	PrintCenterTextAll("%s Flag lost! (%N)", g_sFlagRead[iCarriedFlagColor], client);
	CPrintToChatAll("%s%s%s{green} flag lost%s! ({default}%N{green})", PREFIX, sTeamColorTag, g_sFlagRead[iCarriedFlagColor], sLastPlaceName, client);
	
	SetEntProp(client, Prop_Data, "m_iFrags", GetClientFrags(client)-GetConVarInt(g_hCVScoreForDropFlag));
	CPrintToChat(client, "%sYou lost{olive} %d{green} point for dropping a flag!", PREFIX, GetConVarInt(g_hCVScoreForDropFlag));
	
	// Reset player's screen fade
	PerformFade(client, 1, {0,0,0,0}, FFADE_IN|FFADE_PURGE);
	
	Stats_Drop(client);
	
	LogPlayerEvent(client, "triggered", "smctf_flag_dropped");
	
	// Return it instantly to the spawn
	if(GetConVarBool(g_hCVReturnOnDrop))
	{
		if(iCarriedFlagColor == BLUE_FLAG)
			EmitSoundToAll(SOUND_BLUE_FLAG_RETURN);
		else
			EmitSoundToAll(SOUND_RED_FLAG_RETURN);
		
		PrintCenterTextAll("%s Flag returned automatically!", g_sFlagRead[iCarriedFlagColor]);
		CPrintToChatAll("%s%s%s{green} flag returned automatically!", PREFIX, sTeamColorTag, g_sFlagRead[iCarriedFlagColor]);

		ResetFlag(iCarriedFlagColor);
	}
	// Setup the dropped flag
	else if(IsValidEntity(g_iFlagEntity[iCarriedFlagColor]))
	{
		new Float:fOrigin[3];
		GetClientAbsOrigin(client, fOrigin);
		
		fOrigin[2] += 10.0;
		
		TR_TraceRayFilter(fOrigin, Float:{90.0,0.0,0.0}, MASK_PLAYERSOLID, RayType_Infinite, TraceRayNoPlayers, client);
		if (TR_DidHit())
		{
			TR_GetEndPosition(fOrigin);
		}
		
		g_fFlagDropPoint[iCarriedFlagColor] = fOrigin;
		TeleportEntity(g_iFlagEntity[iCarriedFlagColor], fOrigin, NULL_VECTOR, NULL_VECTOR);
		
		// Apply the spawn protection
		SetEntityRenderMode(g_iFlagEntity[iCarriedFlagColor], RENDER_TRANSCOLOR);
		SetEntityRenderColor(g_iFlagEntity[iCarriedFlagColor], 0, 0, 0, 255);
		g_hResetFlagProtection[iCarriedFlagColor] = CreateTimer(GetConVarFloat(g_hCVFlagProtectionTime), Timer_ResetFlagProtection, iCarriedFlagColor, TIMER_FLAG_NO_MAPCHANGE);
		g_iFlagProtectionStartTime[iCarriedFlagColor] = GetTime();
		
		// create laser beam
		new ent = CreateEntityByName("env_beam");
		TeleportEntity(ent, fOrigin, NULL_VECTOR, NULL_VECTOR);
		SetEntityModel(ent, MDL_LASER);
		DispatchKeyValue(ent, "texture", MDL_LASER);
		DispatchKeyValue(ent, "TouchType", "1");
		DispatchKeyValue(ent, "BoltWidth", "10.0");
		DispatchKeyValue(ent, "life", "0");
		DispatchKeyValue(ent, "rendercolor", "0 0 0");
		new String:sTargetName[64], String:sLowColor[10];
		strtolower(g_sFlagRead[iCarriedFlagColor], sLowColor, sizeof(sLowColor));
		Format(sTargetName, sizeof(sTargetName), "sctf_flag_%s_pole_beam", sLowColor);
		DispatchKeyValue(ent, "targetname", sTargetName);
		fOrigin[2] += 90.0;
		SetEntPropVector(ent, Prop_Data, "m_vecEndPos", fOrigin);
		SetEntPropFloat(ent, Prop_Data, "m_fWidth", 10.0);
		AcceptEntityInput(ent, "TurnOn");
		HookSingleEntityOutput(ent, "OnTouchedByEntity", EntOut_OnTouchFlag);
		g_iDropBeam[iCarriedFlagColor] = ent;
		
		g_hResetDroppedFlag[iCarriedFlagColor] = CreateTimer(GetConVarFloat(g_hCVDropResetTime), Timer_ResetDroppedFlag, iCarriedFlagColor, TIMER_FLAG_NO_MAPCHANGE);
		g_iResetDroppedFlagCountdown[iCarriedFlagColor] = GetConVarInt(g_hCVDropResetTime)-1;
		g_hResetDroppedFlagCountdown[iCarriedFlagColor] = CreateTimer(1.0, Timer_ResetDroppedFlagCountdown, iCarriedFlagColor, TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
	}
	
	
}

public bool:TraceRayNoPlayers(entity, mask, any:data)
{
	if(entity == data || entity >= 1 || entity <= MaxClients)
	{
		return false;
	}
	return true;
}

public Hook_OnPMThink(entity)
{
	// show flag carriers on radar
	for(new i=0;i<=65;i++)
	{
		if(g_iPlayerCarriesFlag[BLUE_FLAG] == i
		|| g_iPlayerCarriesFlag[RED_FLAG] == i)
			SetEntData(entity, g_iRadarOffset + i, 1, 4, true);
	}
	
	#if defined USE_HIDDENBOTS
	// Hide hidden bots
	if(g_iHiddenBotClient[BLUE_FLAG] != -1)
	{
		SetEntData(entity, g_iRadarOffset + g_iHiddenBotClient[BLUE_FLAG], 0, 4, true);
		SetEntData(entity, g_ibConnectedOffset + g_iHiddenBotClient[BLUE_FLAG]*4, 0, 4, true);
		SetEntData(entity, g_ibAliveOffset + g_iHiddenBotClient[BLUE_FLAG]*4, 0, 4, true);
	}
	if(g_iHiddenBotClient[RED_FLAG] != -1)
	{
		SetEntData(entity, g_iRadarOffset + g_iHiddenBotClient[RED_FLAG], 0, 4, true);
		SetEntData(entity, g_ibConnectedOffset + g_iHiddenBotClient[RED_FLAG]*4, 0, 4, true);
		SetEntData(entity, g_ibAliveOffset + g_iHiddenBotClient[RED_FLAG]*4, 0, 4, true);
	}
	#endif
}

public Hook_OnPMThinkPost(entity)
{
	// Switch showing the other flag guy on the radar, if both flags are captured
	if(g_iPlayerCarriesFlag[RED_FLAG] != -1 && g_iPlayerCarriesFlag[BLUE_FLAG] != -1)
	{
		if(g_iVIPShowSwitch == RED_FLAG)
			g_iVIPShowSwitch = BLUE_FLAG;
		else
			g_iVIPShowSwitch = RED_FLAG;
	}
	// Show the symbol permanently if only one flag is captured
	else if(g_iPlayerCarriesFlag[RED_FLAG] != -1)
	{
		g_iVIPShowSwitch = RED_FLAG;
	}
	else if(g_iPlayerCarriesFlag[BLUE_FLAG] != -1)
	{
		g_iVIPShowSwitch = BLUE_FLAG;
	}
	
	// Show the flag carrier as VIP on the scoreboard
	if(g_iPlayerCarriesFlag[g_iVIPShowSwitch] != -1)
	{
		SetEntData(entity, g_iPlayerVIPOffset, g_iPlayerCarriesFlag[g_iVIPShowSwitch], 4, true);
	}
	// Don't display anyone, when no flag carried
	else if(g_iPlayerCarriesFlag[BLUE_FLAG] == -1 && g_iPlayerCarriesFlag[RED_FLAG] == -1)
		SetEntData(entity, g_iPlayerVIPOffset, 0, 4, true);
	
	// show flag carriers on radar
	Hook_OnPMThink(entity);
}

public Action:Command_SetFlag(client, args)
{
	if(!client || !IsPlayerAlive(client))
	{
		ReplyToCommand(client, "SM:CTF > You have to be ingame and alive.");
		return Plugin_Handled;
	}
	
	if(args < 1)
	{
		ReplyToCommand(client, "SM:CTF > Usage: sm_setflag <red|blue>");
		return Plugin_Handled;
	}
	
	new String:sColor[6];
	GetCmdArg(1, sColor, sizeof(sColor));
	
	new iColor;
	if(StrEqual(sColor, "red"))
		iColor = RED_FLAG;
	else if(StrEqual(sColor, "blue"))
		iColor = BLUE_FLAG;
	else
	{
		ReplyToCommand(client, "SM:CTF > Usage: sm_setflag <red|blue>");
		return Plugin_Handled;
	}
	
	new Handle:kv = CreateKeyValues("Flags2");
	new String:file[256], String:map[64];
	GetCurrentMap(map, sizeof(map));
	BuildPath(Path_SM, file, 255, "configs/smctf_maps.cfg");
	FileToKeyValues(kv, file);
	
	KvJumpToKey(kv, map, true);
	
	decl String:sOrigin[50], Float:fOrigin[3];

	GetClientAbsOrigin(client, fOrigin);
	Format(sOrigin, sizeof(sOrigin), "%f %f %f", fOrigin[0], fOrigin[1], fOrigin[2]);
	
	KvSetString(kv, sColor, sOrigin);

	g_fFlagSpawnPoint[iColor][0] = fOrigin[0];
	g_fFlagSpawnPoint[iColor][1] = fOrigin[1];
	g_fFlagSpawnPoint[iColor][2] = fOrigin[2];
	
	KvRewind(kv);
	KeyValuesToFile(kv, file);

	// Remove the old flag stuff
	if(g_iFlagEntity[iColor] != -1 && IsValidEntity(g_iFlagEntity[iColor]))
	{
		AcceptEntityInput(g_iFlagEntity[iColor], "Kill");
		g_iFlagEntity[iColor] = -1;
	}
	if(g_iPoleEntity[iColor] != -1 && IsValidEntity(g_iPoleEntity[iColor]))
	{
		AcceptEntityInput(g_iPoleEntity[iColor], "Kill");
		g_iPoleEntity[iColor] = -1;
	}
	
	new String:sTargetName[64];
	strtolower(g_sFlagRead[iColor], sTargetName, sizeof(sTargetName));
	Format(sTargetName, sizeof(sTargetName), "sctf_flag_%s_pole_beam", sTargetName);
	new iPoleBeam = Entity_FindByName(sTargetName, "env_beam");
	if(iPoleBeam != INVALID_ENT_REFERENCE)
	{
		AcceptEntityInput(iPoleBeam, "Kill");
	}
	
	SpawnFlag(iColor);
	ResetFlag(iColor);
	
	CPrintToChat(client, "%s Placed the %s flag at %s.", PREFIX, g_sFlagRead[iColor], sOrigin);
	
	CloseHandle(kv);
	
	return Plugin_Handled;
}

public Action:Command_DropFlag(client, args)
{
	if(args < 1)
	{
		ReplyToCommand(client, "SM:CTF > Usage: sm_dropflag <#userid|steamid|name>");
		return Plugin_Handled;
	}
	
	new String:sTarget[50];
	GetCmdArg(1, sTarget, sizeof(sTarget));
	
	new iTarget = FindTarget(client, sTarget, false, false);
	if(iTarget == -1)
		return Plugin_Handled;
	
	if(g_iPlayerCarriesFlag[BLUE_FLAG] != iTarget
	&& g_iPlayerCarriesFlag[RED_FLAG] != iTarget)
	{
		ReplyToCommand(client, "SM:CTF > %N doesn't carry any flag!", iTarget);
		return Plugin_Handled;
	}
	
	OnFlagDropped(iTarget);
	
	return Plugin_Handled;
}

// Return the flag automatically after x seconds after drop
public Action:Timer_ResetDroppedFlag(Handle:timer, any:color)
{
	if(!g_bFlagDropped[color])
	{
		g_hResetDroppedFlag[color] = INVALID_HANDLE;
		return Plugin_Stop;
	}
	
	ResetFlag(color);
	
	if(color == BLUE_FLAG)
		EmitSoundToAll(SOUND_BLUE_FLAG_RETURN);
	else
		EmitSoundToAll(SOUND_RED_FLAG_RETURN);
	
	new String:sTeamColorTag[10];
	strtolower(g_sFlagRead[color], sTeamColorTag, sizeof(sTeamColorTag));
	Format(sTeamColorTag, sizeof(sTeamColorTag), "{%s}", sTeamColorTag);
	PrintCenterTextAll("%s Flag returned automatically!", g_sFlagRead[color]);
	CPrintToChatAll("%s%s%s{green} flag returned automatically after {default}%d{green} seconds!", PREFIX, sTeamColorTag, g_sFlagRead[color], GetConVarInt(g_hCVDropResetTime));
	
	return Plugin_Stop;
}

// Return the flag automatically after x seconds after drop
public Action:Timer_ResetDroppedFlagCountdown(Handle:timer, any:color)
{
	g_iResetDroppedFlagCountdown[color]--;
	if(g_iResetDroppedFlagCountdown[color] <= 0 || !g_bFlagDropped[color])
	{
		g_iResetDroppedFlagCountdown[color] = 0;
		g_hResetDroppedFlagCountdown[color] = INVALID_HANDLE;
		return Plugin_Stop;
	}
	return Plugin_Continue;
}

public Action:Timer_DrawStatusPanel(Handle:timer, any:data)
{
	new String:sStatusText[256];
	new iScoreLimit = GetConVarInt(g_hCVScoreLimit);
	// Red flag
	Format(sStatusText, sizeof(sStatusText), "%s Flag: ", g_sFlagRead[RED_FLAG]);
	if(g_bFlagDropped[RED_FLAG])
		Format(sStatusText, sizeof(sStatusText), "%sDropped (%ds)\n", sStatusText, g_iResetDroppedFlagCountdown[RED_FLAG]);
	else if(g_iPlayerCarriesFlag[RED_FLAG] != -1)
	{
		Format(sStatusText, sizeof(sStatusText), "%s%N", sStatusText, g_iPlayerCarriesFlag[RED_FLAG]);
		new iTeam = GetClientTeam(g_iPlayerCarriesFlag[RED_FLAG]);
		if(iTeam == CS_TEAM_T)
			Format(sStatusText, sizeof(sStatusText), "%s (T)\n", sStatusText);
		else if(iTeam == CS_TEAM_CT)
			Format(sStatusText, sizeof(sStatusText), "%s (CT)\n", sStatusText);
	}
	else
		Format(sStatusText, sizeof(sStatusText), "%sIdle\n", sStatusText);
	Format(sStatusText, sizeof(sStatusText), "%s%s Team Score: %d/%d\n \n", sStatusText, g_sFlagRead[RED_FLAG], g_iTeamScore[RED_FLAG], iScoreLimit);
	
	// Blue flag
	Format(sStatusText, sizeof(sStatusText), "%s%s Flag: ", sStatusText, g_sFlagRead[BLUE_FLAG]);
	if(g_bFlagDropped[BLUE_FLAG])
		Format(sStatusText, sizeof(sStatusText), "%sDropped (%ds)\n", sStatusText, g_iResetDroppedFlagCountdown[BLUE_FLAG]);
	else if(g_iPlayerCarriesFlag[BLUE_FLAG] != -1)
	{
		Format(sStatusText, sizeof(sStatusText), "%s%N", sStatusText, g_iPlayerCarriesFlag[BLUE_FLAG]);
		new iTeam = GetClientTeam(g_iPlayerCarriesFlag[BLUE_FLAG]);
		if(iTeam == CS_TEAM_T)
			Format(sStatusText, sizeof(sStatusText), "%s (T)\n", sStatusText);
		else if(iTeam == CS_TEAM_CT)
			Format(sStatusText, sizeof(sStatusText), "%s (CT)\n", sStatusText);
	}
	else
		Format(sStatusText, sizeof(sStatusText), "%sIdle\n", sStatusText);
	Format(sStatusText, sizeof(sStatusText), "%s%s Team Score: %d/%d", sStatusText, g_sFlagRead[BLUE_FLAG], g_iTeamScore[BLUE_FLAG], iScoreLimit);
	
	PrintHintTextToAll(sStatusText);
	
	return Plugin_Continue;
}

public Action:Timer_PlayFlagEffects(Handle:timer, any:data)
{
	if(g_iFlagEntity[BLUE_FLAG] != -1 
	&& IsValidEntity(g_iFlagEntity[BLUE_FLAG])
	&& (g_bFlagDropped[BLUE_FLAG]
	|| g_iPlayerCarriesFlag[BLUE_FLAG] == -1))
	{
		new Float:fSpawnPoint[3], Float:fEndPoint[3];
		if(g_bFlagDropped[BLUE_FLAG])
			fSpawnPoint = g_fFlagDropPoint[BLUE_FLAG];
		else
			fSpawnPoint = g_fFlagSpawnPoint[BLUE_FLAG];
		
		fEndPoint = fSpawnPoint;
		fSpawnPoint[2] += 10.0;
		
		// Show a green beam, if the flag is still protected
		new iBeamColor[4];
		if(g_hResetFlagProtection[BLUE_FLAG] != INVALID_HANDLE || g_bRoundEnded)
			iBeamColor = {0,255,0,255};
		// Show beam in flag color and play other effects
		else
		{
			TE_SetupEnergySplash(fSpawnPoint, Float:{0.0,0.0,0.0}, false);
			TE_SendToAll();
			TE_SetupMetalSparks(fSpawnPoint, Float:{0.0,0.0,0.0});
			TE_SendToAll();
			TE_SetupGlowSprite(fSpawnPoint, g_iGlowSprite, 0.95, 1.5, 50);
			TE_SendToAll();
			iBeamColor = {0,255,255,255};
		}
		
		fSpawnPoint[2] += 8192.0;
		TE_SetupBeamPoints(fSpawnPoint, fEndPoint, g_iBeamSprite, g_iHaloSprite, 0, 20, 1.0, 
                30.0, 100.0, 1, 3.0, iBeamColor, 10);
		TE_SendToAll();
	}
	
	if(g_iFlagEntity[RED_FLAG] != -1 
	&& IsValidEntity(g_iFlagEntity[RED_FLAG])
	&& (g_bFlagDropped[RED_FLAG]
	|| g_iPlayerCarriesFlag[RED_FLAG] == -1))
	{
		new Float:fSpawnPoint[3], Float:fEndPoint[3];
		if(g_bFlagDropped[RED_FLAG])
			fSpawnPoint = g_fFlagDropPoint[RED_FLAG];
		else
			fSpawnPoint = g_fFlagSpawnPoint[RED_FLAG];
		fSpawnPoint[2] += 10.0;
		
		fEndPoint = fSpawnPoint;
		
		// Show a green beam, if the flag is still protected
		new iBeamColor[4];
		if(g_hResetFlagProtection[RED_FLAG] != INVALID_HANDLE || g_bRoundEnded)
			iBeamColor = {0,255,0,255};
		// Show beam in flag color and play other effects
		else
		{
			TE_SetupEnergySplash(fSpawnPoint, Float:{0.0,0.0,0.0}, false);
			TE_SendToAll();
			TE_SetupMetalSparks(fSpawnPoint, Float:{0.0,0.0,0.0});
			TE_SendToAll();
			TE_SetupGlowSprite(fSpawnPoint, g_iGlowSprite, 0.95, 1.5, 50);
			TE_SendToAll();
			iBeamColor = {255,0,0,255};
		}
		
		fSpawnPoint[2] += 8192.0;
		TE_SetupBeamPoints(fSpawnPoint, fEndPoint, g_iBeamSprite, g_iHaloSprite, 0, 20, 1.0, 
                30.0, 100.0, 1, 3.0, iBeamColor, 10);
		TE_SendToAll();
	}
	
	return Plugin_Continue;
}

public Action:Timer_CreateBeamFollow(Handle:timer, any:color)
{
	if(g_iPlayerCarriesFlag[color] == -1 || g_iSpeechIcon[color] == -1)
	{
		g_hBeamFollowTimer[color] = INVALID_HANDLE;
		return Plugin_Stop;
	}
	
	new iColorArray[4];
	iColorArray = g_iFlagRGBColor[color];
	iColorArray[3] = 200;
	TE_SetupBeamFollow(g_iSpeechIcon[color], g_iBeamSprite, g_iHaloSprite, 1.0, 40.0, 10.0, 1, iColorArray);
	TE_SendToAll();
	
	g_hBeamFollowTimer[color] = INVALID_HANDLE;
	
	return Plugin_Stop;
}

public Action:Timer_ResetFlagProtection(Handle:timer, any:color)
{
	g_hResetFlagProtection[color] = INVALID_HANDLE;
	g_iFlagProtectionStartTime[color] = -1;
	SetEntityRenderMode(g_iFlagEntity[color], RENDER_TRANSCOLOR);
	SetEntityRenderColor(g_iFlagEntity[color], 255, 255, 255, 255);
	
	return Plugin_Stop;
}

ParseFlagSpawns()
{
	new Handle:kv = CreateKeyValues("Flags2");
	new String:file[256], String:map[64];
	GetCurrentMap(map, sizeof(map));
	BuildPath(Path_SM, file, 255, "configs/smctf_maps.cfg");
	FileToKeyValues(kv, file);
	
	if (!KvJumpToKey(kv, map))
	{
		SetFailState("Can't find flagconfig for map %s.", map);
		return;
	}
	
	decl String:sFlagCoords[30], String:sVectors[3][16];

	KvGetString(kv, "red", sFlagCoords, sizeof(sFlagCoords));
	TrimString(sFlagCoords);
	ExplodeString(sFlagCoords, " ", sVectors, 3, 16);

	g_fFlagSpawnPoint[RED_FLAG][0] = StringToFloat(sVectors[0]);
	g_fFlagSpawnPoint[RED_FLAG][1] = StringToFloat(sVectors[1]);
	g_fFlagSpawnPoint[RED_FLAG][2] = StringToFloat(sVectors[2]);
	
	KvGetString(kv, "blue", sFlagCoords, sizeof(sFlagCoords));
	TrimString(sFlagCoords);
	ExplodeString(sFlagCoords, " ", sVectors, 3, 16);

	g_fFlagSpawnPoint[BLUE_FLAG][0] = StringToFloat(sVectors[0]);
	g_fFlagSpawnPoint[BLUE_FLAG][1] = StringToFloat(sVectors[1]);
	g_fFlagSpawnPoint[BLUE_FLAG][2] = StringToFloat(sVectors[2]);
	
	CloseHandle(kv);
}

SpawnFlag(color)
{
	ResetFlag(color);
	
	// Spawn the pole
	new iPole = CreateEntityByName("prop_dynamic");
	if(!IsValidEntity(iPole))
	{
		SetFailState("Can't create pole.");
		return;
	}
	TeleportEntity(iPole, g_fFlagSpawnPoint[color], NULL_VECTOR, NULL_VECTOR);
	SetEntityModel(iPole, "models/mapmodels/pole.mdl");
	DispatchSpawn(iPole);
	
	SetEntProp(iPole, Prop_Send, "m_clrRender", -1);
	new String:sTargetName[64], String:sLowColor[10];
	strtolower(g_sFlagRead[color], sLowColor, sizeof(sLowColor));
	Format(sTargetName, sizeof(sTargetName), "sctf_flag_%s_pole", sLowColor);
	DispatchKeyValue(iPole, "targetname", sTargetName);
	DispatchKeyValue(iPole, "rendercolor", "0 0 0");
	
	g_iPoleEntity[color] = iPole;
	
	// create laser beam
	new ent = CreateEntityByName("env_beam");
	TeleportEntity(ent, g_fFlagSpawnPoint[color], NULL_VECTOR, NULL_VECTOR);
	SetEntityModel(ent, MDL_LASER);
	DispatchKeyValue(ent, "texture", MDL_LASER);
	DispatchKeyValue(ent, "TouchType", "1");
	DispatchKeyValue(ent, "BoltWidth", "10.0");
	DispatchKeyValue(ent, "life", "0");
	DispatchKeyValue(ent, "rendercolor", "0 0 0");
	Format(sTargetName, sizeof(sTargetName), "sctf_flag_%s_pole_beam", sLowColor);
	DispatchKeyValue(ent, "targetname", sTargetName);
	new Float:fPos[3];
	fPos = g_fFlagSpawnPoint[color];
	fPos[2] += 90.0;
	SetEntPropVector(ent, Prop_Data, "m_vecEndPos", fPos);
	SetEntPropFloat(ent, Prop_Data, "m_fWidth", 10.0);
	AcceptEntityInput(ent, "TurnOn");
	HookSingleEntityOutput(ent, "OnTouchedByEntity", EntOut_OnTouchPole);
	
	
	// Spawn the flag
	new iFlag = CreateEntityByName("prop_dynamic");
	if(!IsValidEntity(iFlag))
	{
		SetFailState("Can't create flag.");
		return;
	}
	TeleportEntity(iFlag, g_fFlagSpawnPoint[color], NULL_VECTOR, NULL_VECTOR);
	SetEntityModel(iFlag, "models/mapmodels/flags.mdl");
	if(color == BLUE_FLAG)
		SetEntProp(iFlag, Prop_Send, "m_nBody", 0);
	else
		SetEntProp(iFlag, Prop_Send, "m_nBody", 3);
	DispatchSpawn(iFlag);
	
	SetEntProp(iFlag, Prop_Send, "m_clrRender", -1);
	Format(sTargetName, sizeof(sTargetName), "sctf_flag_%s", sLowColor);
	DispatchKeyValue(iFlag, "targetname", sTargetName);
	SetVariantString("flag_idle1");
	AcceptEntityInput(iFlag, "SetAnimation");
	
	g_iFlagEntity[color] = iFlag;
	
	ResetFlag(color);
}

CreateSpeechIcon(client)
{
	if(g_bRoundEnded || (g_iPlayerCarriesFlag[RED_FLAG] != client && g_iPlayerCarriesFlag[BLUE_FLAG] != client))
		return;
	
	// Spawn the speech icon
	new iSpeechIcon = CreateEntityByName("prop_dynamic");
	if(!IsValidEntity(iSpeechIcon))
	{
		SetFailState("Can't create speech icon.");
		return;
	}
	
	new color;
	if(g_iPlayerCarriesFlag[RED_FLAG] == client)
		color = RED_FLAG;
	else
		color = BLUE_FLAG;
	
	new String:sBuffer[64], String:sLowColor[10];
	strtolower(g_sFlagRead[color], sLowColor, sizeof(sLowColor));
	
	new Float:fOrigin[3];
	GetClientAbsOrigin(client, fOrigin);
	fOrigin[2] += 96;
	
	TeleportEntity(iSpeechIcon, fOrigin, NULL_VECTOR, NULL_VECTOR);
	
	Format(sBuffer, sizeof(sBuffer), "models/extras/sctf/info_speech_%sflag.mdl", sLowColor);
	SetEntityModel(iSpeechIcon, sBuffer);
	DispatchSpawn(iSpeechIcon);
	
	SetEntProp(iSpeechIcon, Prop_Send, "m_clrRender", -1);

	Format(sBuffer, sizeof(sBuffer), "sctf_%s_flag_carrier", sLowColor);
	DispatchKeyValue(iSpeechIcon, "targetname", sBuffer);
	
	SetVariantString("!activator");
	AcceptEntityInput(iSpeechIcon, "SetParent", client);
	
	SetVariantString("idle");
	AcceptEntityInput(iSpeechIcon, "SetAnimation");
	
	DispatchKeyValue(iSpeechIcon, "solid", "0");
	
	new iColorArray[4];
	iColorArray = g_iFlagRGBColor[color];
	iColorArray[3] = 200;
	TE_SetupBeamFollow(iSpeechIcon, g_iBeamSprite, g_iHaloSprite, 1.0, 40.0, 10.0, 1, iColorArray);
	TE_SendToAll();
	
	g_iSpeechIcon[color] = iSpeechIcon;
}

ResetFlag(color)
{
	if(g_iSpeechIcon[color] != -1 && IsValidEntity(g_iSpeechIcon[color]))
	{
		AcceptEntityInput(g_iSpeechIcon[color], "Kill");
		g_iSpeechIcon[color] = -1;
	}
	
	if(g_iPlayerCarriesFlag[color] != -1 && IsClientInGame(g_iPlayerCarriesFlag[color]))
	{
		PerformFade(g_iPlayerCarriesFlag[color], 1, {0,0,0,0}, FFADE_IN|FFADE_PURGE);
	}
	
	g_iPlayerCarriesFlag[color] = -1;
	g_bFlagDropped[color] = false;
	
	if(g_iDropBeam[color] != -1 && IsValidEntity(g_iDropBeam[color]))
	{
		AcceptEntityInput(g_iDropBeam[color], "Kill");
		g_iSpeechIcon[color] = -1;
	}
	
	if(g_hBeamFollowTimer[color] != INVALID_HANDLE)
	{
		KillTimer(g_hBeamFollowTimer[color]);
		g_hBeamFollowTimer[color] = INVALID_HANDLE;
	}
	
	g_iResetDroppedFlagCountdown[color] = 0;
	g_fFlagDropPoint[color][0] = 0.0;
	g_fFlagDropPoint[color][1] = 0.0;
	g_fFlagDropPoint[color][2] = 0.0;
	
	if(g_hResetDroppedFlag[color] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlag[color]);
		g_hResetDroppedFlag[color] = INVALID_HANDLE;
	}
	
	if(g_hResetDroppedFlagCountdown[color] != INVALID_HANDLE)
	{
		KillTimer(g_hResetDroppedFlagCountdown[color]);
		g_hResetDroppedFlagCountdown[color] = INVALID_HANDLE;
	}
	
	g_iFlagProtectionStartTime[color] = -1;
	if(g_hResetFlagProtection[color] != INVALID_HANDLE)
	{
		KillTimer(g_hResetFlagProtection[color]);
		g_hResetFlagProtection[color] = INVALID_HANDLE;
	}
	
	if(g_iFlagEntity[color] != -1 && IsValidEntity(g_iFlagEntity[color]))
	{
		TeleportEntity(g_iFlagEntity[color], g_fFlagSpawnPoint[color], NULL_VECTOR, NULL_VECTOR);
		
		// Apply the spawn protection
		SetEntityRenderMode(g_iFlagEntity[color], RENDER_TRANSCOLOR);
		SetEntityRenderColor(g_iFlagEntity[color], 0, 0, 0, 255);
		g_hResetFlagProtection[color] = CreateTimer(GetConVarFloat(g_hCVFlagProtectionTime), Timer_ResetFlagProtection, color, TIMER_FLAG_NO_MAPCHANGE);
		g_iFlagProtectionStartTime[color] = GetTime();
	}

}