/*
 * Handles the player stats
 * Part of SM:CTF
 *
 * visit http://www.wcfan.de/
 */
#include <sourcemod>
#include <sdktools>
#include <cstrike>

new Handle:g_hDatabase = INVALID_HANDLE;

public Action:Command_Stats(client, args)
{
	if(!client)
	{
		ReplyToCommand(client, "SM:CTF > You have to be ingame.");
		return Plugin_Handled;
	}
	
	if(g_hDatabase == INVALID_HANDLE)
	{
		ReplyToCommand(client, "SM:CTF > No database connection. Stats disabled.");
		return Plugin_Handled;
	}
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "SELECT num_spawns, num_capture, num_drop, num_return, num_score, num_win, num_lose, num_draw, lastconnect FROM wcfanctf_stats WHERE authid = '%s';", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ShowStatsPanel, sQuery, client);
 
	return Plugin_Handled;

}

public Query_ShowStatsPanel(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if(hndl == INVALID_HANDLE || strlen(error) > 0)
	{
		LogError("Error retrieving player stats, %s", error);
	}
	
	new Handle:hPanel = CreatePanel();
	SetPanelTitle(hPanel, "SM:CTF Stats");
	
	decl String:sBuffer[64];
	// num_spawns, num_capture, num_drop, num_return, num_score, num_win, num_lose, num_draw
	while(SQL_MoreRows(hndl))
	{
		if(SQL_FetchRow(hndl))
		{
			Format(sBuffer, sizeof(sBuffer), "Spawns: %d", SQL_FetchInt(hndl, 0));
			DrawPanelItem(hPanel, sBuffer);
			
			Format(sBuffer, sizeof(sBuffer), "Captured Flags: %d", SQL_FetchInt(hndl, 1));
			DrawPanelItem(hPanel, sBuffer);
			
			Format(sBuffer, sizeof(sBuffer), "Dropped Flags: %d", SQL_FetchInt(hndl, 2));
			DrawPanelItem(hPanel, sBuffer);
			
			Format(sBuffer, sizeof(sBuffer), "Returned Flags: %d", SQL_FetchInt(hndl, 3));
			DrawPanelItem(hPanel, sBuffer);
			
			Format(sBuffer, sizeof(sBuffer), "Scored Flags: %d", SQL_FetchInt(hndl, 4));
			DrawPanelItem(hPanel, sBuffer);
			
			Format(sBuffer, sizeof(sBuffer), "Wins: %d", SQL_FetchInt(hndl, 5));
			DrawPanelItem(hPanel, sBuffer);
			
			Format(sBuffer, sizeof(sBuffer), "Loses: %d", SQL_FetchInt(hndl, 6));
			DrawPanelItem(hPanel, sBuffer);
			
			Format(sBuffer, sizeof(sBuffer), "Draws: %d", SQL_FetchInt(hndl, 7));
			DrawPanelItem(hPanel, sBuffer);
			
			break;
		}
	}
 
	SendPanelToClient(hPanel, client, Panel_Stats, 20);
 
	CloseHandle(hPanel);
}

public Panel_Stats(Handle:menu, MenuAction:action, param1, param2)
{
	/* Do nothing in here */
}

public Action:Command_ResetStats(client, args)
{
	if(args < 1)
	{
		ReplyToCommand(client, "SM:CTF > Usage: sm_resetctfstats <#userid|steamid|name>");
		return Plugin_Handled;
	}
	
	new String:sTarget[50];
	GetCmdArg(1, sTarget, sizeof(sTarget));
	
	new iTarget = FindTarget(client, sTarget, false, false);
	if(iTarget == -1)
		return Plugin_Handled;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(iTarget, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_spawns = 0, num_capture = 0, num_drop = 0, num_return = 0, num_score = 0, num_win = 0, num_lose = 0, num_draw = 0, lastconnect = 0 WHERE authid = '%s';", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
	
	ReplyToCommand(client, "SM:CTF > Resetted %N's stats.", iTarget);
	
	return Plugin_Handled;
}

/**
 * 
 * SQL Logging
 * 
 */ 

InitializeDatabase()
{
	decl String:sError[256];
	g_hDatabase = SQLite_UseDatabase("wcfanctf_stats", sError, sizeof(sError));
	if(g_hDatabase == INVALID_HANDLE)
	{
		LogError("Error opening sqlite database wcfanctf_stats, %s", sError);
		return;
	}
	
	decl String:sQuery[512];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS wcfanctf_stats (authid VARCHAR(32) NOT NULL, num_spawns INTEGER DEFAULT 0, num_capture INTEGER DEFAULT 0, num_drop INTEGER DEFAULT 0, num_return INTEGER DEFAULT 0, num_score INTEGER DEFAULT 0, num_win INT DEFAULT 0, num_lose INTEGER DEFAULT 0, num_draw INTEGER DEFAULT 0, lastconnect INTEGER DEFAULT 0, PRIMARY KEY (authid))");
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

// Initialize the player in the database
Stats_JoinInsert(client)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "INSERT OR IGNORE INTO wcfanctf_stats (authid) VALUES ('%s');", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

Stats_Spawn(client)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_spawns = num_spawns + 1 WHERE authid = '%s';", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

Stats_Capture(client)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_capture = num_capture + 1 WHERE authid = '%s';", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

Stats_Drop(client)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_drop = num_drop + 1 WHERE authid = '%s';", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

Stats_Return(client)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_return = num_return + 1 WHERE authid = '%s';", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

Stats_Score(client)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_score = num_score + 1 WHERE authid = '%s';", sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

Stats_Connect(client)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	GetClientAuthString(client, sAuthId, sizeof(sAuthId));
	Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET lastconnect = %d WHERE authid = '%s';", GetTime(), sAuthId);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

Stats_Win(team)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i) && GetClientTeam(i) == team)
		{
			GetClientAuthString(i, sAuthId, sizeof(sAuthId));
			Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_win = num_win + 1 WHERE authid = '%s';", sAuthId);
			SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
		}
	}
}

Stats_Lose(team)
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i) && GetClientTeam(i) == team)
		{
			GetClientAuthString(i, sAuthId, sizeof(sAuthId));
			Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_lose = num_lose + 1 WHERE authid = '%s';", sAuthId);
			SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
		}
	}
}

Stats_Draw()
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512], String:sAuthId[32];
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i) && (GetClientTeam(i) == CS_TEAM_CT || GetClientTeam(i) == CS_TEAM_T))
		{
			GetClientAuthString(i, sAuthId, sizeof(sAuthId));
			Format(sQuery, sizeof(sQuery), "UPDATE wcfanctf_stats SET num_draw = num_draw + 1 WHERE authid = '%s';", sAuthId);
			SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
		}
	}
}

Stats_PruneOld()
{
	if(g_hDatabase == INVALID_HANDLE)
		return;
	
	decl String:sQuery[512];
	Format(sQuery, sizeof(sQuery), "DELETE FROM wcfanctf_stats WHERE (%d - lastconnect) >= %d;", GetTime(), GetConVarInt(g_hCVStatPrune)*60);
	SQL_TQuery(g_hDatabase, Query_ErrorCallback, sQuery);
}

public Query_ErrorCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if(hndl == INVALID_HANDLE || strlen(error) > 0)
	{
		LogError("Error excuting database query, %s", error);
	}
}